package com.bettersoftwarefaster.carserv.app;

import java.util.*;
import javax.swing.*;
import com.bettersoftwarefaster.carserv.infra.*;

/**
 * Async. equivalent of AppActionCommand.
 * <p>Listener of own completion, as a convenience.
 */
public abstract class AppActionAsyncCommand extends AppActionCommand
implements CommandCompletionListener {
    public AppActionAsyncCommand(String name) {
        this(name, Command.ExecReturn.NOT_UNDOABLE);
    }

    public AppActionAsyncCommand(String name, Command.ExecReturn execReturn) {
        super(name);
        initialize(execReturn);
    }

    public AppActionAsyncCommand(String name, Icon icon) {
        this(name, icon, Command.ExecReturn.NOT_UNDOABLE);
    }

    public AppActionAsyncCommand(String name, Icon icon,
    Command.ExecReturn execReturn) {
        super(name, icon);
        initialize(execReturn);
    }

    public AppActionAsyncCommand(String name, char mnemonic) {
        this(name, mnemonic, Command.ExecReturn.NOT_UNDOABLE);
    }

    public AppActionAsyncCommand(String name, char mnemonic,
    Command.ExecReturn execReturn) {
        super(name, mnemonic);
        initialize(execReturn);
    }

    public AppActionAsyncCommand(String name, Icon icon, char mnemonic) {
        this(name, icon, mnemonic, Command.ExecReturn.NOT_UNDOABLE);
    }

    public AppActionAsyncCommand(String name, Icon icon, char mnemonic,
    Command.ExecReturn execReturn) {
        super(name, icon, mnemonic);
        initialize(execReturn);
    }

    private void initialize(Command.ExecReturn execReturn) {
        this.execReturn = execReturn;
        addCommandCompletionListener(this);
    }

    private Command.ExecReturn execReturn = Command.ExecReturn.NOT_UNDOABLE; // belt and braces

    public void addCommandCompletionListener(CommandCompletionListener ccl) {
        listeners.add(ccl);
    }

    public void removeCommandCompletionListener(CommandCompletionListener ccl) {
        listeners.remove(ccl);
    }

    /**
     * Replaces the synchronous implementation of execute() inherited with
     * an async. equivalent.  The pre- and post- hooks are handled
     * slightly differently.
     */
    public final Command.ExecReturn execute(final Object arg)
    throws CommandException {
        if (arg == null) {
            return Command.ExecReturn.IGNORE;
        }
        if (isExecuting()) {
            return Command.ExecReturn.IGNORE;
        }
        this.arg = arg;
        preExecute(arg);
        setEnabled(false);
        this.thread = new Thread() {
            public void run() {
                try {
                    doExecute(arg);
                }
                catch (CommandException ex) {
                    commandException = ex;
                }
                for (Iterator iter = listeners.iterator(); iter.hasNext(); ) {
                    ((CommandCompletionListener)iter.next()).
                    commandCompleted(AppActionAsyncCommand.this,
                    commandException);
                }
            }
        };
        thread.start();
        return execReturn;
    }

    /**
     * Implementation of the actual command.  The return status is ignored;
     * instead the required return code should be passed into the contructor.
     */
    protected abstract Command.ExecReturn doExecute(Object arg)
    throws CommandException;

    public boolean isExecuting() {
        return (thread != null && thread.isAlive());
    }

    public CommandException getException() { return commandException; }

    /**
     * This is final to ensure that the semantics of the hook methods
     * defined in the superclass continue to be honoured.
     */
    public final void commandCompleted(Command command, CommandException ex) {
        setEnabled(true);
        if (ex == null) {
            postExecute(arg);
        }
        else {
            executeFailed(ex);
        }
    }

    public Thread getThread() { return thread; }

    private CommandException commandException;
    private Thread thread;
    private Object arg;

    /**
     * @link aggregation
     *   @associates <{CommandCompletionListener}>
     * @clientCardinality 1
     * @supplierCardinality 0..*
     * @constraint {ordered}
     */
    private List listeners = new LinkedList();
}
