package com.bettersoftwarefaster.carserv.app;

import java.util.*;
import com.bettersoftwarefaster.carserv.infra.*;
import javax.swing.*;
import java.awt.event.ActionEvent;

/**
 * Ties together carserv.infra.Command interface and Swing's Action interface.
 * <p>To use, implement doActionPerformed() and return arg parameter so that
 * the execute(arg) method can be invoked. If the action is undoable, also
 * override undo(). <P>The command itself is invoked through the (singleton)
 * CommandManager so that it can be undone/redone (if supported).
 */
public abstract class AppActionCommand extends AppAction implements Command {
    public AppActionCommand(String name) {
        super(name);
    }

    public AppActionCommand(String name, Icon icon) {
        super(name, icon);
    }

    public AppActionCommand(String name, char mnemonic) {
        super(name, mnemonic);
    }

    public AppActionCommand(String name, Icon icon, char mnemonic) {
        super(name, icon, mnemonic);
    }

    public final void actionPerformed(ActionEvent ev) {
        Object arg = getArg(ev);
        this.arg = arg;
        try {
            getCommandManager().execute(this, arg);
        }
        catch (CommandException ex) {
            // do nothing, processed in execute() itself
        }
    }

    public void executeSelf() {
        actionPerformed(null);
    }

    public CommandManager getCommandManager() {
        return AppSingletonManager.getInstance().getCommandManager();
    }

    public String toString() {
        return "AppActionCommand: " + this.getValue(Action.NAME);
    }

    /**
     * @pattern template
     * @patternrole hook
     * @returns argument to be passed to execute()
     */
    protected abstract Object getArg(ActionEvent ev);

    /**
     * should <b>not</b> be overridden; use doExecute() instead
     * <p>implementation note: is not marked as final just so that it can be
     * overridden by AppActionAsyncCommand
     * @see AppActionAsyncCommand
     */
    public Command.ExecReturn execute(Object arg) throws CommandException {
        if (arg == null) {
            return Command.ExecReturn.IGNORE;
        }
        preExecute(arg);
        Command.ExecReturn execReturn;
        try {
            execReturn = doExecute(arg);
        }
        catch (CommandException ex) {
            executeFailed(ex);
            throw ex;
        }
        postExecute(arg);
        return execReturn;
    }

    protected void preExecute(Object arg) { }

    protected void postExecute(Object arg) { }

    public void executeFailed(CommandException ex) {
        System.err.println(getName() + ": execute failed" + "\n  arg = " +
        arg + "\n  ex = " + ex);
    }

    /** Implementation of the actual command */
    protected abstract Command.ExecReturn doExecute(Object arg)
    throws CommandException;

    public final Command.UndoReturn undo(Object arg) throws CommandException {
        preUndo(arg);
        Command.UndoReturn undoReturn;
        try {
            undoReturn = doUndo(arg);
        }
        catch (CommandException ex) {
            undoFailed(ex);
            throw ex;
        }
        postUndo(arg);
        return undoReturn;
    }

    protected void preUndo(Object arg) { }

    /** Implementation of the undo; by default not undoable but can override. */
    public Command.UndoReturn doUndo(Object arg) throws CommandException {
        throw new CommandException("Undo not available.");
    }

    public void undoFailed(CommandException ex) {
        System.err.println(getName() + ": undo failed" + "\n  arg = " + arg +
        "\n  ex = " + ex);
    }

    protected void postUndo(Object arg) { }

    private Object arg;
}
