package com.bettersoftwarefaster.carserv.app;

import java.util.*;
import com.bettersoftwarefaster.carserv.infra.*;
import com.bettersoftwarefaster.carserv.domain.*;

/**
 * Manager of all singletons for the 'app' package.  Represents a single-point
 * to obtain type-safe references to any singletons in the 'app' package.
 * Also provides access to the corresponding DomainSingletonManager in the
 * 'domain' package, thus giving access to any singletons in that package.
 * <p>Implementation notes: creates the SingletonRegister and binds all its
 * singletons into it.  Also instantiates the DomainSingletonManager, passing
 * the SingletonRegister to the DomainSingletonManager also.  Note that both
 * this class and the DomainSingletonManager are themselves singletons.
 * @see DomainSingletonManager
 */
public class AppSingletonManager {
    /**
     * Access to AppSingletonManager, itself a Singleton.
     * This saves us having to pass the thing around to the constructor of
     * every class in this package.
     */
    public static AppSingletonManager getInstance() {
        if (instance == null) {
            throw new IllegalStateException("Not yet initialised.");
        }
        return instance;
    }

    /** @supplierRole instance */
    private static AppSingletonManager instance;

    public AppSingletonManager(CarServApp app, SingletonRegister singletonRegister)
    throws AppException {
        this.singletonRegister = singletonRegister;
        bind("CarServApp", app);
        bindManagers();
        bindDomainSingletonManager(); // needs to be last, uses getter's
        bind("AppSingletonManager", this); // (not actually used)
        instance = this; // for getInstance()
    }

    private void bindManagers() throws AppException {
        this.commandManager = new CommandManager();
        bind("CommandManager", commandManager);
    }

    private void bindDomainSingletonManager() {
        this.domainSingletonManager =
        new DomainSingletonManager(singletonRegister);
        bind("DomainSingletonManager", domainSingletonManager);
    }

    private void bind(String name, Object object) {
        singletonRegister.bind(name, object);
    }

    private Object get(String name) {
        return singletonRegister.get(name);
    }

    public DomainSingletonManager getDomainSingletonManager() {
        return (DomainSingletonManager)get("DomainSingletonManager");
    }

    public CarServApp getApp() {
        return (CarServApp)get("CarServApp");
    }

    public CommandManager getCommandManager() {
        return (CommandManager)get("CommandManager");
    }

    /**
     * @supplierCardinality 1
     * @label uses
     */
    private final SingletonRegister singletonRegister;

    /**
     * @label creates
     * @supplierCardinality 1
     */
    private DomainSingletonManager domainSingletonManager;

    /**
     * @supplierCardinality 1
     * @label creates
     */
    private CommandManager commandManager;
}
