package com.bettersoftwarefaster.carserv.app;

import java.util.*;
import java.sql.*;
import com.bettersoftwarefaster.carserv.infra.*;
import com.bettersoftwarefaster.carserv.domain.*;
import com.bettersoftwarefaster.carserv.datamgmt.*;

/**
 * Represents the logical application.  Singleton.  However, services provided
 * by the logical application are invoked only through the
 * AppActionCommand subclasses.
 * @stereotype System
 * @pattern Singleton
 */
public class CarServApp {
    public static CarServApp getInstance() {
        if (instance == null) {
            throw new IllegalStateException("Not yet initialised.");
        }
        return instance;
    }

    /** @supplierRole instance */
    private static CarServApp instance;

    public CarServApp() throws AppException {
        this.singletonRegister = new SingletonRegister();
        this.appSingletonManager =
        new AppSingletonManager(this, singletonRegister);
        instance = this;
    }

    /** @stereotype systemEvent */
    public Employee signOn(String lastName, String firstName)
    throws AppException {
        Employee employee;
        try {
            employee = getSessionManager().signOn(lastName, firstName);
        }
        catch (DomainException ex) {
            throw new AppException("Unable to sign on", ex);
        }
        return employee;
    }

    /** @stereotype systemEvent */
    public void logout() {
        getSessionManager().logout();
    }

    public boolean isLoggedIn() {
        return getSessionManager().isLoggedIn();
    }

    public Employee getCurrentUser() {
        return getSessionManager().getCurrentUser();
    }

    /**
     * @stereotype systemEvent
     * @link dependency
     *   @associates <{Customer}>
     */
    public List listCustomers(String lastName, String firstName,
    String addressStreet, String addressPostalCode) throws AppException {
        List customers;
        try {
            customers = getCustomerFactory().listCustomers(lastName, firstName,
            addressStreet, addressPostalCode);
        }
        catch (DomainException ex) {
            throw new AppException("Unable to list customers", ex);
        }
        return customers;
    }

    /** @stereotype systemEvent */
    public List listServices(java.sql.Date date) throws AppException {
        ServiceData[] serviceData;
        List services;
        try {
            serviceData = getServiceDM().listServices(date);
            services = new ArrayList();
            for (int i = 0; i < serviceData.length; i++) {
                services.add(getServiceHelper().serviceFor(serviceData[i]));
            }
        }
        catch (DataMgmtException ex) {
            throw new AppException("Problem listing services", ex);
        }
        return services;
    }

    private CustomerFactory getCustomerFactory() {
        return appSingletonManager.getDomainSingletonManager().getCustomerFactory();
    }

    private ServiceDM getServiceDM() {
        return appSingletonManager.getDomainSingletonManager().getServiceDM();
    }

    private ServiceHelper getServiceHelper() {
        return appSingletonManager.getDomainSingletonManager().getServiceHelper();
    }

    private DomainSessionManager getSessionManager() {
        return appSingletonManager.getDomainSingletonManager().getDomainSessionManager();
    }

    public AppSingletonManager getAppSingletonManager() {
        return appSingletonManager;
    }

    public SingletonRegister getSingletonRegister() {
        return singletonRegister;
    }

    /**
     * @link aggregationByValue
     * @supplierCardinality 1
     * @label creates
     */
    final private AppSingletonManager appSingletonManager;

    /**
     * @supplierCardinality 1
     * @label creates
     */
    final private SingletonRegister singletonRegister;
}
