package com.bettersoftwarefaster.carserv.datamgmt;

import java.util.*;
import java.sql.*;

/**
 * Designed as a superclasses for DataManagers - classes that need to interact
 * with the database.  It is itself a subclass of DatabaseAdapter, from which
 * it inherits transaction management functionality, but also implements
 * DatabaseSessionListener, so is aware of when a database session is or
 * is not available.
 */
public abstract class DataManager extends DatabaseAdapter implements
DatabaseSessionListener {
    DataManager() {
    }

    // TODO: reinstate protected later
    //	protected Connection getConnection() throws DataMgmtException {
    public Connection getConnection() throws DataMgmtException {
        if (getDatabaseSession() == null) {
            throw new DatabaseNotAvailableException("No session");
        }
        Connection connection = getDatabaseSession().getConnection();
        try {
            if (connection == null || connection.isClosed()) {
                throw new DataMgmtException("An open connection is required.");
            }
        }
        catch (SQLException ex) {
            throw new IllegalArgumentException("An open connection is required - SQLException occurred.");
        }
        return connection;
    }

    protected boolean isNotEmpty(String param) {
        return (param != null && param.length() != 0);
    }

    protected String isNull(String param, String ifNull) {
        return (param == null) ? ifNull : param;
    }

    protected String wildCard(String param) {
        return isNull(param, "") + "%";
    }

    protected String wildCard2(String param) {
        if (param == null)
            param = "";
        return "%" + isNull(param, "") + "%";
    }

    public void sessionAvailable(DatabaseSessionEvent ev) {
        databaseSession = ev.getDatabaseSession();
        hookSessionAvailable();
    }

    public void sessionUnavailable(DatabaseSessionEvent ev) {
        databaseSession = null;
        hookSessionUnavailable();
    }

    protected abstract void hookSessionAvailable();

    protected abstract void hookSessionUnavailable();

    public DatabaseSession getDatabaseSession() {
        return databaseSession;
    }

    public boolean isSessionAvailable() {
        return (databaseSession != null);
    }

    private DatabaseSession getValidatedDatabaseSession()
    throws DatabaseNotAvailableException {
        if (!isSessionAvailable()) {
            throw new DatabaseNotAvailableException("No session.");
        }
        return databaseSession;
    }

    protected void beginTransaction() throws DataMgmtException {
        getValidatedDatabaseSession().beginTransaction();
    }

    protected void rollbackTransaction() throws DataMgmtException {
        getValidatedDatabaseSession().rollbackTransaction();
    }

    protected void commitTransaction() throws DataMgmtException {
        getValidatedDatabaseSession().commitTransaction();
    }

    protected PreparedStatement getPreparedStatement(String sql)
    throws DataMgmtException {
        return getValidatedDatabaseSession().getPreparedStatement(sql);
    }

    private DatabaseSession databaseSession;
}
