package com.bettersoftwarefaster.carserv.datamgmt;

import java.util.*;
import java.sql.*;

/**
 * Connection to backend database for some user.  The user
 * information is validated against the database; in the future an LDAP or
 * other non-proprietary authentication service could be substituted.
 */
public class DatabaseSession extends DatabaseAdapter {

    //    public final static String DB_TYPE = "local";
    public final static String DB_TYPE = "remote";

    public static void main(String[] args) {
        try {
            DatabaseSession session = new DatabaseSession();
            System.out.println("instantiated ok");
        }
        catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    /**
     * sets autocommit true for the connection.  This is disabled whenever a
     * transaction is started.
     * @see DatabaseAdapter for transaction control
     */
    public DatabaseSession() throws DataMgmtException {
        this.connectionFactory =
        ConnectionFactory.getConnectionFactory(DB_TYPE);
        this.connection = connectionFactory.getConnection();
        StringBuffer buf = new StringBuffer();
        buf.append("\n SELECT emp_id, emp_mch_labour_rate");
        buf.append("\n FROM   emp_employee");
        buf.append("\n WHERE  emp_last_name  = ?");
        buf.append("\n AND    emp_first_name = ?");
        this.signOnPS = getPreparedStatement(buf.toString());
    }

    public EmployeeData signOn(String employeeLastName, String employeeFirstName)
    throws DataMgmtException {

        checkConnection();
        ResultSet resultSet = null;
        int numRows = 0;
        try {
            signOnPS.clearParameters();
            signOnPS.setString(1, employeeLastName);
            signOnPS.setString(2, employeeFirstName);
            resultSet = signOnPS.executeQuery();
            while (resultSet.next()) {
                this.user = new EmployeeData(resultSet.getInt(1),
                employeeLastName, employeeFirstName, resultSet.getLong(2));
                numRows++;
            }
            resultSet.close();
        }
        catch (SQLException ex) {
            logout();
            throw new DataMgmtException("Failed to signOn", ex);
        }
        if (numRows == 0) {
            logout();
            throw new DataMgmtException("Unable to locate user record (in employee table).");
        }
        if (numRows > 1) {
            logout();
            throw new DataMgmtException("Duplicate user records (in employee table) found!");
        }
        return getUser();
    }

    public void logout() {
        this.user = null;
        if (this.connection != null) {
            try {
                connection.close();
            }
            catch (SQLException ex) { }
        }
        connection = null;
    }

    public EmployeeData getUser() throws DatabaseNotAvailableException {
        checkConnection();
        return this.user;
    }

    public Connection getConnection() throws DatabaseNotAvailableException {
        checkConnection();
        return this.connection;
    }

    protected void beginTransaction() throws DataMgmtException {
        checkConnection();
        try {
            getConnection().setAutoCommit(false);
        }
        catch (SQLException ex) {
            throw new DataMgmtException("Problem beginning transaction", ex);
        }
    }

    protected void rollbackTransaction() throws DataMgmtException {
        checkConnection();
        try {
            getConnection().rollback();
            getConnection().setAutoCommit(true);
        }
        catch (SQLException ex) {
            throw new DataMgmtException("Problem rolling back transaction", ex);
        }
    }

    protected void commitTransaction() throws DataMgmtException {
        checkConnection();
        try {
            getConnection().commit();
            getConnection().setAutoCommit(true);
        }
        catch (SQLException ex) {
            throw new DataMgmtException("Problem committing transaction", ex);
        }
    }

    protected PreparedStatement getPreparedStatement(String sql)
    throws DataMgmtException {
        checkConnection();
        PreparedStatement statement;
        try {
            statement = getConnection().prepareStatement(sql);
        }
        catch (SQLException ex) {
            throw new DataMgmtException("Unable to prepared statement, sql = " +
            sql, ex);
        }
        return statement;
    }

    private void checkConnection() throws DatabaseNotAvailableException {
        // don't use getConnection(); it calls checkConnection in turn...
        if (!isDatabaseConnectionValid()) {
            throw new DatabaseNotAvailableException("Database not available");
        }
    }

    public boolean isDatabaseConnectionValid() {
        try {
            if ((connection == null) || connection.isClosed()) {
                return false;
            }
        }
        catch (SQLException ex) {
            return false;
        }
        return true;
    }

    /**
     * @link aggregationByValue
     * @supplierCardinality 1
     * @label uses
     */
    private final ConnectionFactory connectionFactory;

    /**
     * @link aggregationByValue
     * @supplierCardinality 1
     * @clientCardinality 1
     * @label uses
     */
    private Connection connection;
    private final PreparedStatement signOnPS;

    /**
     * @supplierRole user
     * @supplierCardinality 0..1
     * @label creates
     */
    private EmployeeData user;
}
