package com.bettersoftwarefaster.carserv.datamgmt;

import java.util.*;

/**
 * Immutable value object to represent a regular service for a Car, based upon
 * its model.  For example, a Ford Focus might have regular services every
 * 7500 miles; this class specifies the parts that need to be changed at some
 * given mileage point.
 * @stereotype valueObject
 */
public class RegularServiceData extends ValueObject {
    public final static long serialVersionUID = 1;

    /**
     * instantiated only by datamgmt
     * @param parts - list of PartData elements
     */
    RegularServiceData(String model, int mileage, List parts) {
        this.model = model;
        this.mileage = mileage;
        this.parts = parts;
    }

    public String getModel() { return model; }

    public int getMileage() { return mileage; }

    /** @returns Collection of PartData elements */
    public List getParts() {
        return Collections.unmodifiableList(parts);
    }

    public boolean equals(Object other) {
        if (!(other instanceof RegularServiceData))
            return false;
        RegularServiceData otherRegularService = (RegularServiceData)other;
        return equalsPK(otherRegularService) &&
        parts.equals(otherRegularService.parts);
    }

    public boolean equalsPK(RegularServiceData otherRegularService) {
        return (otherRegularService == null) ? false :
        compareStrings(model, otherRegularService.model) &&
        mileage == otherRegularService.mileage;
    }

    public int hashCode() {
        return (model + ":" + mileage).hashCode();
    }

    public String toString() {
        return getClass().getName() + ": " + valueOf("model  ", getModel()) +
        valueOf("mileage", "" + getMileage()) +
        valueOf("Embedded parts:\n", parts);
    }

    private final String model;
    private final int mileage;

    /**
     * @link aggregationByValue
     *   @associates <{PartData}>
     * @clientCardinality 1
     * @supplierCardinality 0..*
     * @supplierRole parts
     * @constraint {set}
     */
    private final List parts;
}
