package com.bettersoftwarefaster.carserv.datamgmt;

import java.util.*;
import java.sql.*;

/**
 * Data manager to manage service detail: <li>fill out labour and part detail
 * <li>maintain labour and part detail <li>adjustments
 */
public class ServiceDM extends DataManager {
    public ServiceDM() {
    }

    /**
     * Returns all services on a given date.  The <<mi-detail>> will not
     * be filled out.
     */
    public ServiceData[] listServices(java.sql.Date date)
    throws DataMgmtException {
        List services = new ArrayList();
        ResultSet resultSet = null;
        try {
            getListServicesPS().clearParameters();
            getListServicesPS().setDate(1, date);
            resultSet = getListServicesPS().executeQuery();
            while (resultSet.next()) {
                services.add(getServiceDataHelper().fetchServiceData(resultSet));
            }
            resultSet.close();
        }
        catch (SQLException ex) {
            throw new DataMgmtException("Failed to list Services", ex);
        }
        return (ServiceData[]) services.toArray(
        new ServiceData[] { });
    }

    /**
     * Returns back the same service (refreshed from DB with latest info),
     * but also with <<mi-detail>> filled out.
     */
    public ServiceData getService(ServiceData service)
    throws DataMgmtException {
        ServiceData serviceNoDetail = null;
        ResultSet resultSet = null;
        try {
            getGetServicePS().clearParameters();
            getGetServicePS().setString(1, service.getLicensePlate());
            getGetServicePS().setDate(2, service.getBookedIn());
            resultSet = getGetServicePS().executeQuery();
            while (resultSet.next()) {
                serviceNoDetail = getServiceDataHelper().fetchServiceData(resultSet);
            }
            resultSet.close();
        }
        catch (SQLException ex) {
            throw new DataMgmtException("Failed to get Service", ex);
        }
        ServiceData serviceWithDetail =
        updateServiceWithChargeableItems(
        updateServiceWithLabour(serviceNoDetail));
        return serviceWithDetail;
    }

    private ServiceData updateServiceWithLabour(ServiceData service)
    throws DataMgmtException {
        ServiceData newService;
        ResultSet resultSet;
        Map labour = new HashMap();
        try {
            getFetchServiceLabourPS().clearParameters();
            getFetchServiceLabourPS().setString(1, service.getLicensePlate());
            getFetchServiceLabourPS().setDate(2, service.getBookedIn());
            resultSet = getFetchServiceLabourPS().executeQuery();
            while (resultSet.next()) {
                EmployeeData mechanic =
                new EmployeeData(resultSet.getInt(1), resultSet.getString(2),
                resultSet.getString(3), resultSet.getLong(4));
                labour.put(mechanic,
                new ServiceData.Labour(mechanic, resultSet.getInt(5),
                resultSet.getString(6)));
            }
            newService = new ServiceData(service.getLicensePlate(), service.getBookedIn(),
            service.getServiceMileage(), service.getActualMileage(), service.getPickedUp(),
            service.getLabourAdjustments(), service.getChargeableItemAdjustments(),
            labour, service.getChargeableItems());
            resultSet.close();
        }
        catch (SQLException ex) {
            throw new DataMgmtException("Failed to fetchServiceLabour", ex);
        }
        return newService;
    }

    private ServiceData updateServiceWithChargeableItems(ServiceData service)
    throws DataMgmtException {
        ServiceData newService;
        ResultSet resultSet;
        Map chargeableItems = new HashMap();
        try {
            getFetchServiceChargeableItemsPS().clearParameters();
            getFetchServiceChargeableItemsPS().setString(1,
            service.getLicensePlate());
            getFetchServiceChargeableItemsPS().setDate(2,
            service.getBookedIn());
            resultSet = getFetchServiceChargeableItemsPS().executeQuery();
            while (resultSet.next()) {
                PartData part = new PartData(resultSet.getInt(1), resultSet.getString(2),
                resultSet.getLong(3));
                chargeableItems.put(part,
                new ServiceData.ChargeableItems(part, resultSet.getInt(4)));
            }
            newService = new ServiceData(service.getLicensePlate(), service.getBookedIn(),
            service.getServiceMileage(), service.getActualMileage(), service.getPickedUp(),
            service.getLabourAdjustments(), service.getChargeableItemAdjustments(),
            service.getLabour(), chargeableItems);
            resultSet.close();
        }
        catch (SQLException ex) {
            throw new DataMgmtException("Failed to fetchServiceChargeableItems", ex);
        }
        return newService;
    }

    public ServiceData bookInCar(ServiceData service, int actualMileage)
    throws DataMgmtException {
        int numRowsUpdated;
        try {
            getBookInCarPS().clearParameters();
            getBookInCarPS().setInt(1, actualMileage);
            getBookInCarPS().setString(2, service.getLicensePlate());
            getBookInCarPS().setDate(3, service.getBookedIn());
            beginTransaction();
            numRowsUpdated = getBookInCarPS().executeUpdate();
        }
        catch (Exception ex) {
            rollbackTransaction();
            throw new DataMgmtException("Failed to book in car", ex);
        }
        if (numRowsUpdated != 1) {
            rollbackTransaction();
            throw new DataMgmtException("Failed to book in car, " +
            numRowsUpdated + " rows updated.");
        }
        else {
            commitTransaction();
        }
        return getService(service);
    }

    public ServiceData pickUpCar(ServiceData service, java.sql.Date pickedUp)
    throws DataMgmtException {
        int numRowsUpdated;
        try {
            getPickedUpCarPS().clearParameters();
            getPickedUpCarPS().setDate(1, pickedUp);
            getPickedUpCarPS().setString(2, service.getLicensePlate());
            getPickedUpCarPS().setDate(3, service.getBookedIn());
            beginTransaction();
            numRowsUpdated = getPickedUpCarPS().executeUpdate();
        }
        catch (Exception ex) {
            rollbackTransaction();
            throw new DataMgmtException("Failed to pick up car", ex);
        }
        if (numRowsUpdated != 1) {
            rollbackTransaction();
            throw new DataMgmtException("Failed to pick up car, " +
            numRowsUpdated + " rows updated.");
        }
        else {
            commitTransaction();
        }
        return getService(service);
    }

    public ServiceData updateLabour(ServiceData service, EmployeeData user,
    int tenthsOfHours, String notes) throws DataMgmtException {
        int numRowsUpdated;
        try {
            beginTransaction();
            if (tenthsOfHours == 0) {
                // delete, don't worry if doesn't find any rows
                getDeleteLabourPS().clearParameters();
                getDeleteLabourPS().setString(1, service.getLicensePlate());
                getDeleteLabourPS().setDate(2, service.getBookedIn());
                getDeleteLabourPS().setInt(3, user.getId());
                numRowsUpdated = getDeleteLabourPS().executeUpdate();
                commitTransaction();
                return getService(service);
            }
            // try to update
            getUpdateLabourPS().clearParameters();
            getUpdateLabourPS().setInt(1, tenthsOfHours);
            getUpdateLabourPS().setString(2, notes);
            getUpdateLabourPS().setString(3, service.getLicensePlate());
            getUpdateLabourPS().setDate(4, service.getBookedIn());
            getUpdateLabourPS().setInt(5, user.getId());
            numRowsUpdated = getUpdateLabourPS().executeUpdate();
            if (numRowsUpdated == 1) {
                commitTransaction();
                return getService(service);
            }
            // doesn't exist, create
            getInsertLabourPS().clearParameters();
            getInsertLabourPS().setString(1, service.getLicensePlate());
            getInsertLabourPS().setDate(2, service.getBookedIn());
            getInsertLabourPS().setInt(3, user.getId());
            getInsertLabourPS().setInt(4, tenthsOfHours);
            getInsertLabourPS().setString(5, notes);
            numRowsUpdated = getInsertLabourPS().executeUpdate();
        }
        catch (Exception ex) {
            rollbackTransaction();
            throw new DataMgmtException("Failed to maintain labour for service", ex);
        }
        if (numRowsUpdated != 1) {
            rollbackTransaction();
            throw new DataMgmtException("Failed to insert labour, " +
            numRowsUpdated + " rows updated.");
        }
        else {
            commitTransaction();
            return getService(service);
        }
    }

    public ServiceData updateChargeableItem(ServiceData service, PartData part,
    int numberOfItems) throws DataMgmtException {
        int numRowsUpdated;
        try {
            beginTransaction();
            if (numberOfItems == 0) {
                // delete, don't worry if doesn't find any rows
                getDeleteChargeableItemsPS().clearParameters();
                getDeleteChargeableItemsPS().setString(1,
                service.getLicensePlate());
                getDeleteChargeableItemsPS().setDate(2, service.getBookedIn());
                getDeleteChargeableItemsPS().setInt(3, part.getNumber());
                numRowsUpdated = getDeleteChargeableItemsPS().executeUpdate();
                commitTransaction();
                return getService(service);
            }
            // try to update
            getUpdateChargeableItemsPS().clearParameters();
            getUpdateChargeableItemsPS().setInt(1, numberOfItems);
            getUpdateChargeableItemsPS().setString(2,
            service.getLicensePlate());
            getUpdateChargeableItemsPS().setDate(3, service.getBookedIn());
            getUpdateChargeableItemsPS().setInt(4, part.getNumber());
            numRowsUpdated = getUpdateChargeableItemsPS().executeUpdate();
            if (numRowsUpdated == 1) {
                commitTransaction();
                return getService(service);
            }
            // doesn't exist, create
            getInsertChargeableItemsPS().clearParameters();
            getInsertChargeableItemsPS().setString(1,
            service.getLicensePlate());
            getInsertChargeableItemsPS().setDate(2, service.getBookedIn());
            getInsertChargeableItemsPS().setInt(3, part.getNumber());
            getInsertChargeableItemsPS().setInt(4, numberOfItems);
            numRowsUpdated = getInsertChargeableItemsPS().executeUpdate();
        }
        catch (Exception ex) {
            rollbackTransaction();
            throw new DataMgmtException("Failed to maintain chargeable item for service", ex);
        }
        if (numRowsUpdated != 1) {
            rollbackTransaction();
            throw new DataMgmtException("Failed to insert chargeable item, " +
            numRowsUpdated + " rows updated.");
        }
        else {
            commitTransaction();
            return getService(service);
        }
    }

    public ServiceData adjustServiceLabour(ServiceData service,
    long labourAdjustments) throws DataMgmtException {
        int numRowsUpdated;
        try {
            getAdjustServiceLabourPS().clearParameters();
            getAdjustServiceLabourPS().setLong(1, labourAdjustments);
            getAdjustServiceLabourPS().setString(2, service.getLicensePlate());
            getAdjustServiceLabourPS().setDate(3, service.getBookedIn());
            beginTransaction();
            numRowsUpdated = getAdjustServiceLabourPS().executeUpdate();
        }
        catch (Exception ex) {
            rollbackTransaction();
            throw new DataMgmtException("Failed to adjust labour costs for service", ex);
        }
        if (numRowsUpdated != 1) {
            rollbackTransaction();
            throw new DataMgmtException("Failed to adjust labour costs for service, " +
            numRowsUpdated + " rows updated.");
        }
        else {
            commitTransaction();
        }
        return getService(service);
    }

    public ServiceData adjustServiceChargeableItems(ServiceData service,
    long chargeableItemAdjustments) throws DataMgmtException {
        int numRowsUpdated;
        try {
            getAdjustServiceChargeableItemPS().clearParameters();
            getAdjustServiceChargeableItemPS().setLong(1,
            chargeableItemAdjustments);
            getAdjustServiceChargeableItemPS().setString(2,
            service.getLicensePlate());
            getAdjustServiceChargeableItemPS().setDate(3,
            service.getBookedIn());
            beginTransaction();
            numRowsUpdated = getAdjustServiceChargeableItemPS().executeUpdate();
        }
        catch (Exception ex) {
            rollbackTransaction();
            throw new DataMgmtException("Failed to adjust chargeable item costs for service", ex);
        }
        if (numRowsUpdated != 1) {
            rollbackTransaction();
            throw new DataMgmtException("Failed to adjust chargeable item costs for service, " +
            numRowsUpdated + " rows updated.");
        }
        else {
            commitTransaction();
        }
        return getService(service);
    }

    private ServiceDataHelper getServiceDataHelper() throws DataMgmtException {
        if (serviceDataHelper == null) {
            serviceDataHelper = new ServiceDataHelper();
        }
        return serviceDataHelper;
    }

    protected void hookSessionAvailable() { }

    protected void hookSessionUnavailable() {
        listServicesPS = null;
        getServicePS = null;
        fetchServiceLabourPS = null;
        fetchServiceChargeableItemsPS = null;
        bookInCarPS = null;
        pickedUpCarPS = null;
        insertLabourPS = null;
        updateLabourPS = null;
        deleteLabourPS = null;
        insertChargeableItemsPS = null;
        updateChargeableItemsPS = null;
        deleteChargeableItemsPS = null;
        adjustServiceLabourPS = null;
        adjustServiceChargeableItemPS = null;
    }

    private PreparedStatement getListServicesPS() throws DataMgmtException {
        if (listServicesPS == null) {
            StringBuffer buf = new StringBuffer();
            buf.append("\n SELECT svc.car_license_plate, svc.svc_booked_in,");
            buf.append("\n        svc.mdl_mileage, svc.svc_mileage, svc.svc_picked_up,");
            buf.append("\n        svc_lbr_adjustments, svc_cim_adjustments");
            buf.append("\n FROM   svc_service svc");
            buf.append("\n WHERE  svc.svc_booked_in = ?");
            buf.append("\n ORDER BY svc.car_license_plate DESC");
            listServicesPS = getPreparedStatement(buf.toString());
        }
        return listServicesPS;
    }

    private PreparedStatement getGetServicePS() throws DataMgmtException {
        if (getServicePS == null) {
            StringBuffer buf = new StringBuffer();
            buf.append("\n SELECT svc.car_license_plate, svc.svc_booked_in,");
            buf.append("\n        svc.mdl_mileage, svc.svc_mileage, svc.svc_picked_up,");
            buf.append("\n        svc_lbr_adjustments, svc_cim_adjustments");
            buf.append("\n FROM   svc_service svc");
            buf.append("\n WHERE  svc.car_license_plate = ?");
            buf.append("\n AND    svc.svc_booked_in     = ?");
            buf.append("\n ORDER BY svc.svc_booked_in DESC");
            getServicePS = getPreparedStatement(buf.toString());
        }
        return getServicePS;
    }

    private PreparedStatement getFetchServiceLabourPS()
    throws DataMgmtException {
        if (fetchServiceLabourPS == null) {
            StringBuffer buf = new StringBuffer();
            buf.append("\n SELECT lbr.emp_id,");
            buf.append("\n        emp.emp_last_name, emp.emp_first_name,");
            buf.append("\n        emp.emp_mch_labour_rate,");
            buf.append("\n        lbr.lbr_hours, lbr.lbr_notes");
            buf.append("\n FROM   svc_service svc");
            buf.append("\n   JOIN lbr_labour lbr");
            buf.append("\n     ON svc.car_license_plate = lbr.car_license_plate");
            buf.append("\n    AND svc.svc_booked_in     = lbr.svc_booked_in");
            buf.append("\n   JOIN emp_employee emp");
            buf.append("\n     ON lbr.emp_id = emp.emp_id");
            buf.append("\n WHERE  svc.car_license_plate = ?");
            buf.append("\n AND    svc.svc_booked_in = ?");
            fetchServiceLabourPS = getPreparedStatement(buf.toString());
        }
        return fetchServiceLabourPS;
    }

    private PreparedStatement getFetchServiceChargeableItemsPS()
    throws DataMgmtException {
        if (fetchServiceChargeableItemsPS == null) {
            StringBuffer buf = new StringBuffer();
            buf.append("\n SELECT cim.prt_num,");
            buf.append("\n        prt.prt_description, prt.prt_cost,");
            buf.append("\n        cim.cim_num_items");
            buf.append("\n FROM   svc_service svc");
            buf.append("\n   JOIN cim_chargeable_item cim");
            buf.append("\n     ON  svc.car_license_plate = cim.car_license_plate");
            buf.append("\n    AND svc.svc_booked_in     = cim.svc_booked_in");
            buf.append("\n   JOIN prt_part prt");
            buf.append("\n     ON cim.prt_num = prt.prt_num");
            buf.append("\n WHERE  svc.car_license_plate = ?");
            buf.append("\n AND    svc.svc_booked_in = ?");
            fetchServiceChargeableItemsPS =
            getPreparedStatement(buf.toString());
        }
        return fetchServiceChargeableItemsPS;
    }

    private PreparedStatement getBookInCarPS() throws DataMgmtException {
        if (bookInCarPS == null) {
            StringBuffer buf = new StringBuffer();
            buf.append("\n UPDATE svc_service");
            buf.append("\n SET    svc_mileage = ?");
            buf.append("\n WHERE  car_license_plate = ?");
            buf.append("\n AND    svc_booked_in = ?");
            bookInCarPS = getPreparedStatement(buf.toString());
        }
        return bookInCarPS;
    }

    private PreparedStatement getPickedUpCarPS() throws DataMgmtException {
        if (pickedUpCarPS == null) {
            StringBuffer buf = new StringBuffer();
            buf.append("\n UPDATE svc_service");
            buf.append("\n SET    svc_picked_up = ?");
            buf.append("\n WHERE  car_license_plate = ?");
            buf.append("\n AND    svc_booked_in = ?");
            pickedUpCarPS = getPreparedStatement(buf.toString());
        }
        return pickedUpCarPS;
    }

    private PreparedStatement getInsertLabourPS() throws DataMgmtException {
        if (insertLabourPS == null) {
            StringBuffer buf = new StringBuffer();
            buf.append("\n INSERT INTO lbr_labour(");
            buf.append("\n        car_license_plate, svc_booked_in,");
            buf.append("\n        emp_id, lbr_hours, lbr_notes    )");
            buf.append("\n VALUES(?, ?, ?, ?, ?)");
            insertLabourPS = getPreparedStatement(buf.toString());
        }
        return insertLabourPS;
    }

    private PreparedStatement getUpdateLabourPS() throws DataMgmtException {
        if (updateLabourPS == null) {
            StringBuffer buf = new StringBuffer();
            buf.append("\n UPDATE lbr_labour");
            buf.append("\n SET    lbr_hours = ?,");
            buf.append("\n        lbr_notes = ?");
            buf.append("\n WHERE  car_license_plate = ?");
            buf.append("\n AND    svc_booked_in = ?");
            buf.append("\n AND    emp_id = ?");
            updateLabourPS = getPreparedStatement(buf.toString());
        }
        return updateLabourPS;
    }

    private PreparedStatement getDeleteLabourPS() throws DataMgmtException {
        if (deleteLabourPS == null) {
            StringBuffer buf = new StringBuffer();
            buf.append("\n DELETE");
            buf.append("\n FROM   lbr_labour");
            buf.append("\n WHERE  car_license_plate = ?");
            buf.append("\n AND    svc_booked_in = ?");
            buf.append("\n AND    emp_id = ?");
            deleteLabourPS = getPreparedStatement(buf.toString());
        }
        return deleteLabourPS;
    }

    private PreparedStatement getInsertChargeableItemsPS()
    throws DataMgmtException {
        if (insertChargeableItemsPS == null) {
            StringBuffer buf = new StringBuffer();
            buf.append("\n INSERT INTO cim_chargeable_item(");
            buf.append("\n        car_license_plate, svc_booked_in,");
            buf.append("\n        prt_num, cim_num_items            )");
            buf.append("\n VALUES(?, ?, ?, ?)");
            insertChargeableItemsPS = getPreparedStatement(buf.toString());
        }
        return insertChargeableItemsPS;
    }

    private PreparedStatement getUpdateChargeableItemsPS()
    throws DataMgmtException {
        if (updateChargeableItemsPS == null) {
            StringBuffer buf = new StringBuffer();
            buf.append("\n UPDATE cim_chargeable_item");
            buf.append("\n SET    cim_num_items = ?");
            buf.append("\n WHERE  car_license_plate = ?");
            buf.append("\n AND    svc_booked_in = ?");
            buf.append("\n AND    prt_num = ?");
            updateChargeableItemsPS = getPreparedStatement(buf.toString());
        }
        return updateChargeableItemsPS;
    }

    private PreparedStatement getDeleteChargeableItemsPS()
    throws DataMgmtException {
        if (deleteChargeableItemsPS == null) {
            StringBuffer buf = new StringBuffer();
            buf.append("\n DELETE");
            buf.append("\n FROM   cim_chargeable_item");
            buf.append("\n WHERE  car_license_plate = ?");
            buf.append("\n AND    svc_booked_in = ?");
            buf.append("\n AND    prt_num = ?");
            deleteChargeableItemsPS = getPreparedStatement(buf.toString());
        }
        return deleteChargeableItemsPS;
    }

    private PreparedStatement getAdjustServiceLabourPS()
    throws DataMgmtException {
        if (adjustServiceLabourPS == null) {
            StringBuffer buf = new StringBuffer();
            buf.append("\n UPDATE svc_service");
            buf.append("\n SET    svc_lbr_adjustments = ?");
            buf.append("\n WHERE  car_license_plate = ?");
            buf.append("\n AND    svc_booked_in = ?");
            adjustServiceLabourPS = getPreparedStatement(buf.toString());
        }
        return adjustServiceLabourPS;
    }

    private PreparedStatement getAdjustServiceChargeableItemPS()
    throws DataMgmtException {
        if (adjustServiceChargeableItemPS == null) {
            StringBuffer buf = new StringBuffer();
            buf.append("\n UPDATE svc_service");
            buf.append("\n SET    svc_cim_adjustments = ?");
            buf.append("\n WHERE  car_license_plate = ?");
            buf.append("\n AND    svc_booked_in = ?");
            adjustServiceChargeableItemPS =
            getPreparedStatement(buf.toString());
        }
        return adjustServiceChargeableItemPS;
    }

    private PreparedStatement listServicesPS;
    private PreparedStatement getServicePS;
    private PreparedStatement fetchServiceLabourPS;
    private PreparedStatement fetchServiceChargeableItemsPS;
    private PreparedStatement bookInCarPS;
    private PreparedStatement pickedUpCarPS;
    private PreparedStatement insertLabourPS;
    private PreparedStatement updateLabourPS;
    private PreparedStatement deleteLabourPS;
    private PreparedStatement insertChargeableItemsPS;
    private PreparedStatement updateChargeableItemsPS;
    private PreparedStatement deleteChargeableItemsPS;
    private PreparedStatement adjustServiceLabourPS;
    private PreparedStatement adjustServiceChargeableItemPS;

    /**
     * @clientCardinality 1
     * @supplierCardinality 1
     * @link aggregationByValue
     */
    private ServiceDataHelper serviceDataHelper;
}
