package com.bettersoftwarefaster.carserv.datamgmt;

import java.sql.*;
import java.util.*;

/**
 * Immutable value object to represent a Service of a Car.  Can distinguish
 * two types of ServiceData's: if serviceMileage == 0, then this is an adhoc
 * rather than a regular service.
 */
public class ServiceData extends ValueObject {
    public final static long serialVersionUID = 1;

    /**
     * @param serviceMileage - set to 0 if ADHC service
     * @param actualMileage  - set to 0 if service only
     * scheduled, not yet done
     * @param pickedUp       - set to null if service only
     * scheduled, not yet done
     * @param labour is map of ServiceData.Labour objects,
     * keyed by mechanic's EmployeeData
     * @param chargeableItems is map of ServiceData.ChargeableItem objects,
     * keyed by part's PartData
     */
    ServiceData(String licensePlate, java.sql.Date bookedIn,
    int serviceMileage, int actualMileage, java.sql.Date pickedUp,
    long labourAdjustments, long chargeableItemAdjustments, Map labour,
    Map chargeableItems) {
        this.licensePlate = licensePlate; // always defined
        this.bookedIn = bookedIn; // always defined
        this.serviceMileage = serviceMileage; // defined for RGLR services
        // defined if service has been completed
        this.actualMileage = actualMileage;
        if (pickedUp != null) {
            this.pickedUp = pickedUp;
        }
        this.labourAdjustments = labourAdjustments;
        this.chargeableItemAdjustments = chargeableItemAdjustments;
        this.labour = (labour == null) ? Collections.EMPTY_MAP : labour;
        this.chargeableItems = (chargeableItems == null) ?
        Collections.EMPTY_MAP : chargeableItems;
    }

    public String getLicensePlate() { return licensePlate; }

    public java.sql.Date getBookedIn() { return bookedIn; }

    /** @returns serviceMileage; 0 for ADHC services */
    public int getServiceMileage() { return serviceMileage; }

    /** @returns actualMileage; 0 if service has not been performed yet */
    public int getActualMileage() { return actualMileage; }

    /** @returns date picked up; null if service has not been performed yet */
    public java.sql.Date getPickedUp() { return pickedUp; }

    /**
     * @returns adjustments to labour charges;
     * 0 if service has not been performed yet or were no adjustments
     */
    public long getLabourAdjustments() { return labourAdjustments; }

    /**
     * @returns adjustments to chargeable items;
     * 0 if service has not been performed yet or were no adjustments
     */
    public long getChargeableItemAdjustments() {
        return chargeableItemAdjustments;
    }

    public Map getLabour() { return Collections.unmodifiableMap(labour); }

    public Map getChargeableItems() {
        return Collections.unmodifiableMap(chargeableItems);
    }

    private String licensePlate;
    private java.sql.Date bookedIn;
    private int serviceMileage;
    private int actualMileage;
    private java.sql.Date pickedUp;
    private long labourAdjustments;
    private long chargeableItemAdjustments;

    /** Doesn't check that labour and chargeable items are necessarily equal */
    public boolean equals(Object other) {
        if (!(other instanceof ServiceData))
            return false;
        ServiceData otherService = (ServiceData)other;
        return equalsPK(otherService) && serviceMileage == otherService.serviceMileage &&
        actualMileage == otherService.actualMileage &&
        compareDates(pickedUp, otherService.pickedUp) &&
        labourAdjustments == otherService.labourAdjustments &&
        chargeableItemAdjustments == otherService.chargeableItemAdjustments &&
        compareMaps(labour, otherService.labour) &&
        compareMaps(chargeableItems, otherService.chargeableItems);
    }

    public boolean equalsPK(ServiceData otherService) {
        //	System.out.println("ServiceData.equalsPK(); compareDates(bookedIn,
        // otherService.bookedIn) returns " + compareDates(bookedIn, otherService.bookedIn) );
        //	System.out.println("ServiceData.equalsPK(); this.bookedIn
        // is " + this.bookedIn         + " .getTime() is " + this.bookedIn.getTime() );
        //	System.out.println("ServiceData.equalsPK(); otherService.bookedIn
        // is " + otherService.bookedIn + " .getTime() is " +
        // otherService.bookedIn.getTime() );
        return (otherService == null) ? false :
        compareStrings(licensePlate, otherService.licensePlate) &&
        compareDates(bookedIn, otherService.bookedIn);
    }

    public int hashCode() {
        return (getLicensePlate() + ":" + bookedIn.toString()).hashCode();
    }

    public String toString() {
        return getClass().getName() + ": " +
        valueOf("license plate             ", getLicensePlate()) +
        valueOf("bookedIn                  ", getBookedIn().toString()) +
        valueOf("serviceMileage            ", getServiceMileage()) +
        valueOf("actualMileage             ", getActualMileage()) +
        valueOf("pickedUp                  ", getPickedUp()) +
        valueOf("labourAdjustments         ", getLabourAdjustments()) +
        valueOf("chargeableItemAdjustments ", getChargeableItemAdjustments()) +
        valueOf("labour (embedded)         ", getLabour()) +
        valueOf("chargeableItems (embedded)", getChargeableItems());
    }

    /**
     * @link aggregationByValue
     *   @associates <{Labour}>
     *   @clientQualifier mechanic
     * @clientCardinality 1
     * @supplierCardinality 0..*
     * @supplierRole labour
     * @constraint {set}
     */
    private final Map labour;

    /**
     * @link aggregationByValue
     *   @associates <{ChargeableItems}>
     *   @clientQualifier part
     * @clientCardinality 1
     * @supplierCardinality 0..*
     * @supplierRole labour
     * @constraint {set}
     */
    private final Map chargeableItems;

    public static class Labour extends ValueObject {
        Labour(EmployeeData mechanic, int hours, String notes) {
            this.mechanic = mechanic;
            this.hours = hours;
            this.notes = notes;
        }

        public EmployeeData getMechanic() { return mechanic; }

        public int getHours() { return hours; }

        public String getNotes() { return notes; }

        private final EmployeeData mechanic;
        private final int hours;
        private final String notes;

        public boolean equals(Object other) {
            if (!(other instanceof Labour))
                return false;
            Labour otherLabour = (Labour)other;
            return mechanic.equals(otherLabour.getMechanic()) &&
            hours == otherLabour.hours &&
            compareStrings(notes, otherLabour.notes);
        }

        public int hashCode() {
            return mechanic.hashCode();
        }

        public String toString() {
            return "  \nLabour:" +
            valueOf("  mechanic.id", getMechanic().getId()) +
            valueOf("  hours      ", getHours()) +
            valueOf("  notes      ", getNotes());
        }
    }

    public static class ChargeableItems extends ValueObject {
        ChargeableItems(PartData part, int numberOfItems) {
            this.part = part;
            this.numberOfItems = numberOfItems;
        }

        public PartData getPart() { return part; }

        public int getNumberOfItems() { return numberOfItems; }

        private final PartData part;
        private final int numberOfItems;

        public boolean equals(Object other) {
            if (!(other instanceof ChargeableItems))
                return false;
            ChargeableItems otherChargeableItems = (ChargeableItems)other;
            return part.equals(otherChargeableItems.getPart()) &&
            numberOfItems == otherChargeableItems.numberOfItems;
        }

        public int hashCode() {
            return part.hashCode();
        }

        public String toString() {
            return "  \nChargeable Item:" +
            valueOf("  part.number     ", getPart().getNumber()) +
            valueOf("  number of items ", getNumberOfItems());
        }
    }
}
