package com.bettersoftwarefaster.carserv.domain;

import java.util.*;
import com.bettersoftwarefaster.carserv.datamgmt.*;
import java.awt.Color;

/**
 * Reference object representing a Car owned by some car owner (= customer).
 * <p>Implementation note: if request services, the mi-detail for services
 * will not be filled out unless request specifically with fillOutService().
 * If refreshServices(), then mi-detail will be lost.
 * @stereotype thing
 * @represents specification
 * @see CarFactory
 */
public class Car {
    /**
     * if don't know car owner
     * @workaround no need to be public, however DocGen blows up
     * (build 1340v6)
     */
    public Car(CarData data) throws DomainException {
        this.model = getModelFactory().getModel(data.getModel(), data);
        this.data = data;
        refreshServices();
    }

    /**
     * if know car owner
     * @workaround no need to be public, however DocGen blows up
     * (build 1340v6)
     */
    public Car(Customer owner, CarData data) throws DomainException {
        this(data);
        this.owner = owner;
    }

    public Model getModel() {
        return model;
    }

    public String getLicensePlate() {
        return data.getLicensePlate();
    }

    public Color getColour() {
        return data.getColour();
    }

    public Customer getOwner() throws DomainException {
        if (owner == null) {
            Customer customer = getCustomerFactory().getCustomer(data.getOwnerId());
            this.owner = customer;
        }
        return owner;
    }

    public List services() {
        List servicesAsList = new ArrayList();
        for (Iterator iter = this.services.values().iterator();
        iter.hasNext(); ) {
            servicesAsList.add(iter.next());
        }
        return Collections.unmodifiableList(servicesAsList);
    }

    public void refreshServices() throws DomainException {
        ServiceData[] serviceData;
        try {
            serviceData = getCarDM().listServices(getData());
        }
        catch (DatabaseNotAvailableException ex) {
            throw new DomainException("Not signed on.", ex);
        }
        catch (DataMgmtException ex) {
            throw new DomainException("Unable to get services for car", ex);
        }
        services.clear();
        for (int i = 0; i < serviceData.length; i++) {
            services.put(serviceData[i].getBookedIn(),
            getServiceHelper().serviceFor(serviceData[i]));
        }
    }

    /**
     * By default, service mi-detail will not be filled out.  Can request for
     * it to be filled out using this method, either using this method or with
     * equivalent method in the service itself.  Further, if interact with
     * service (eg update it's labour)
     * then the mi-detail will again be filled out.
     */
    public Service fillOutService(java.sql.Date serviceDate)
    throws DomainException {
        Service service = (Service)services.get(serviceDate);
        if (service == null) {
            throw new DomainException("No service on specified date, " +
            serviceDate);
        }
        service.fillOut();
        return service;
    }

    public RegularService scheduleRegularService(java.sql.Date bookedIn,
    int serviceMileage) throws DomainException {
        if (!this.model.isValidServiceMileage(serviceMileage)) {
            throw new DomainException("Service mileage is incorrect for this model.");
        }
        try {
            ServiceData serviceData =
            getCarDM().scheduleRegularService(getData(),
            bookedIn, serviceMileage);
            return (RegularService)putService(serviceData);
        }
        catch (DatabaseNotAvailableException ex) {
            throw new DomainException("Not signed on.", ex);
        }
        catch (DataMgmtException ex) {
            throw new DomainException("Unable to schedule regular service", ex);
        }
    }

    public AdhocService scheduleAdhocService(java.sql.Date bookedIn)
    throws DomainException {
        try {
            ServiceData serviceData = getCarDM().scheduleAdhocService(getData(), bookedIn);
            return (AdhocService)putService(serviceData);
        }
        catch (DatabaseNotAvailableException ex) {
            throw new DomainException("Not signed on.", ex);
        }
        catch (DataMgmtException ex) {
            throw new DomainException("Unable to schedule adhoc service", ex);
        }
    }

    private Service putService(ServiceData serviceData) {
        Service service = getServiceHelper().serviceFor(serviceData);
        services.put(serviceData.getBookedIn(), service);
        return service;
    }

    public void cancelService(Service service) throws DomainException {
        if (!services.containsKey(service.getBookedIn())) {
            throw new DomainException("No such service");
        }
        try {
            ServiceData serviceData = service.getData();
            getCarDM().cancelService(serviceData);
            services.remove(service.getBookedIn());
        }
        catch (DatabaseNotAvailableException ex) {
            throw new DomainException("Not signed on.", ex);
        }
        catch (DataMgmtException ex) {
            throw new DomainException("Unable to cancel service", ex);
        }
    }

    private CarDM getCarDM() {
        return DomainSingletonManager.getInstance().getCarDM();
    }

    private ServiceHelper getServiceHelper() {
        return DomainSingletonManager.getInstance().getServiceHelper();
    }

    private CustomerFactory getCustomerFactory() {
        return DomainSingletonManager.getInstance().getCustomerFactory();
    }

    private ModelFactory getModelFactory() {
        return DomainSingletonManager.getInstance().getModelFactory();
    }

    public CarData getData() {
        return data;
    }

    public String toString() {
        return getLicensePlate() + " (" + System.identityHashCode(this) + ")";
    }

    /**
     * @clientCardinality 0..*
     * @supplierCardinality 1
     */
    private Model model;

    /**
     * @associates <{com.bettersoftwarefaster.carserv.domain.Service}>
     *   @clientQualifier bookedIn
     * @clientCardinality 1
     * @supplierCardinality 0..*
     * @constraint {set}
     * @constraint {ordered}
     * @bidirectional <{com.bettersoftwarefaster.carserv.domain.Service#car}>
     */
    private Map services = new TreeMap();

    /**
     * @supplierCardinality 1
     * @link aggregationByValue
     * @supplierRole data
     */
    private CarData data;

    /** @bidirectional */
    private Customer owner;
}
