package com.bettersoftwarefaster.carserv.domain;

import java.util.*;
import com.bettersoftwarefaster.carserv.infra.*;
import com.bettersoftwarefaster.carserv.datamgmt.*;
import com.bettersoftwarefaster.carserv.app.AppSingletonManager;

/**
 * Instantiated and provides type-safe access to all domain singletons,
 * physically stored in a SingletonRegister, passed into the constructor.  Is
 * itself a singleton.
 * @see AppSingletonManager
 */
public class DomainSingletonManager {
    /**
     * Access to DomainSingletonManager, itself a Singleton.
     * This saves us having to pass the thing around to the constructor of
     * every class in this package.
     */
    public static DomainSingletonManager getInstance() {
        if (instance == null) {
            throw new IllegalStateException("Not yet initialised.");
        }
        return instance;
    }

    /** @supplierRole instance */
    private static DomainSingletonManager instance;

    public DomainSingletonManager(SingletonRegister singletonRegister) {
        this.singletonRegister = singletonRegister;
        bindSingletonFactories();
        bindSingletonHelpers();
        bindSingletonDatabaseManagers();
        bind("DomainSingletonManager", this); // (not actually used)
        instance = this; // for getInstance()
        bindDomainSessionManager(); // needs to be last, uses getter's
    }

    private void bindDomainSessionManager() {
        this.domainSessionManager = new DomainSessionManager();
        bind("DomainSessionManager", domainSessionManager);
    }

    private void bindSingletonFactories() {
        this.carFactory = new CarFactory();
        this.customerFactory = new CustomerFactory();
        this.modelFactory = new ModelFactory();
        this.partFactory = new PartFactory();
        this.employeeFactory = new EmployeeFactory();
        bind("CarFactory", carFactory);
        bind("CustomerFactory", customerFactory);
        bind("ModelFactory", modelFactory);
        bind("PartFactory", partFactory);
        bind("EmployeeFactory", employeeFactory);
    }

    private void bindSingletonHelpers() {
        this.serviceHelper = new ServiceHelper();
        bind("ServiceHelper", serviceHelper);
    }

    void bindSingletonDatabaseManagers() {
        this.carDM = new CarDM();
        this.custDM = new CustDM();
        this.serviceDM = new ServiceDM();
        bind("CarDM", carDM);
        bind("CustDM", custDM);
        bind("ServiceDM", serviceDM);
    }

    private void bind(String name, Object object) {
        singletonRegister.bind(name, object);
    }

    private Object get(String name) {
        return singletonRegister.get(name);
    }

    public DomainSessionManager getDomainSessionManager() {
        return (DomainSessionManager)get("DomainSessionManager");
    }

    public CarFactory getCarFactory() {
        return (CarFactory)get("CarFactory");
    }

    public CustomerFactory getCustomerFactory() {
        return (CustomerFactory)get("CustomerFactory");
    }

    public ModelFactory getModelFactory() {
        return (ModelFactory)get("ModelFactory");
    }

    public PartFactory getPartFactory() {
        return (PartFactory)get("PartFactory");
    }

    public EmployeeFactory getEmployeeFactory() {
        return (EmployeeFactory)get("EmployeeFactory");
    }

    public ServiceHelper getServiceHelper() {
        return (ServiceHelper)get("ServiceHelper");
    }

    public CustDM getCustDM() {
        return (CustDM)get("CustDM");
    }

    public ServiceDM getServiceDM() {
        return (ServiceDM)get("ServiceDM");
    }

    public CarDM getCarDM() {
        return (CarDM)get("CarDM");
    }

    /**
     * @label creates
     * @supplierCardinality 1
     * @link aggregation
     */
    private DomainSessionManager domainSessionManager;
    private PartFactory partFactory;
    private ModelFactory modelFactory;
    private CustomerFactory customerFactory;
    private CarFactory carFactory;
    private EmployeeFactory employeeFactory;
    private ServiceHelper serviceHelper;
    private CarDM carDM;
    private ServiceDM serviceDM;
    private CustDM custDM;

    /**
     * @supplierCardinality 1
     * @label uses
     */
    private final SingletonRegister singletonRegister;
}
