package com.bettersoftwarefaster.carserv.domain;

import com.bettersoftwarefaster.carserv.datamgmt.*;
import java.util.*;

/**
 * Implementation class that maintains a Map of Models, to
 * ensure that reference
 * object semantics of Model is maintained.  <p>Trusts clients,
 * in the sense that
 * clients can provide Model details to the ModelFactory and the factory will
 * remember them if that Model is previously unknown.  (This design is to
 * reduce to-ing and fro-ing to the back-end database).  In fact, totally
 * relies on clients to populate its map, so any design that uses ModelFactory
 * must ensure that it will have been correctly populated previously
 * before calling.
 * @stereotype factory
 * @see Model
 */
public class ModelFactory extends FactoryAdapter {
    ModelFactory() {
    }

    public Model getModel(String name) throws DomainException {
        Model model = (Model)getMap().get(name);
        if (model == null) {
            throw new DomainException("No such model.");
        }
        return model;
    }

    /**
     * @param name - name of model to fetch
     * @param data - underlying data if model not previously known
     */
    public Model getModel(String name, CarData data) throws DomainException {
        String modelName = data.getModel();
        Model model = (Model)getMap().get(modelName);
        if (model == null) {
            try {
                RegularServiceData[] regularServiceData =
                getCarDM().listRegularServices(data);
                model = new Model(modelName, regularServiceData);
                getMap().put(modelName, model);
            }
            catch (DatabaseNotAvailableException ex) {
                throw new DomainException("Not signed on.");
            }
            catch (DataMgmtException ex) {
                throw new DomainException("Unable to retrieve regular services for model "
                + modelName, ex);
            }
        }
        return model;
    }

    private CarDM getCarDM() {
        return DomainSingletonManager.getInstance().getCarDM();
    }
}
