package com.bettersoftwarefaster.carserv.domain;

import com.bettersoftwarefaster.carserv.datamgmt.*;
import java.util.*;

/**
 * Abstraction of Service, representing for example an adhoc or a regular
 * service (as defined by the manufacturer).
 * <p>Unlike other domain objects, does not have full reference object
 * semantics; there can be more than one instance of Service representing - at
 * the end of the day - the same row in the database.
 * @stereotype moment-interval
 * @represents specification
 * @see ServiceHelper
 */
public abstract class Service {
    /** if car is not known */
    Service(ServiceData data) {
        reset(data);
    }

    private void reset(ServiceData data) {
        this.data = data;
    }

    /** if car is known */
    Service(Car car, ServiceData data) {
        this(data);
        this.car = car;
    }

    /**
     * The car returned knows nothing of this Service; if ask to list its
     * services, then the list will not contain this service, though there
     * will be a service that is semantically the same as this.
     * <p>It was a design decision not to implement Service as a valueObject;
     * hence comparing the two services semantically the same
     * using equals() will
     * <b>not</b> return true. <p>(Another way of saying all this is that the
     * implied referential integrity
     * link between Car and Service is not enforced).
     */
    public Car getCar() throws DomainException {
        if (car == null) {
            car = getCarFactory().getCar(getLicensePlate());
        }
        return car;
    }

    public void bookInCar(int mileage) throws DomainException {
        try {
            ServiceData data = getServiceDM().bookInCar(getData(), mileage);
            reset(data);
        }
        catch (DatabaseNotAvailableException ex) {
            throw new DomainException("Not signed on.", ex);
        }
        catch (DataMgmtException ex) {
            throw new DomainException("Unable to book in car", ex);
        }
        getCar().refreshServices();
        // since RI not maintained, need to tell the car
        // its update its version of the services
    }

    public void fillOut() throws DomainException {
        try {
            ServiceData serviceData = getServiceDM().getService(getData());
            reset(serviceData);
        }
        catch (DatabaseNotAvailableException ex) {
            throw new DomainException("Not signed on.", ex);
        }
        catch (DataMgmtException ex) {
            throw new DomainException("Unable to fill out service detail", ex);
        }
    }

    /** will fill out <<mi-detail>> if not already filled out */
    public void updateLabour(int tenthsOfHours, String notes)
    throws DomainException {
        Employee employee = getSessionManager().getCurrentUser();
        try {
            ServiceData data = getServiceDM().
            updateLabour(getData(), employee.getData(), tenthsOfHours, notes);
            reset(data);
        }
        catch (DatabaseNotAvailableException ex) {
            throw new DomainException("Not signed on.", ex);
        }
        catch (DataMgmtException ex) {
            throw new DomainException("Unable to update labour", ex);
        }
    }

    /** will fill out <<mi-detail>> if not already filled out */
    public void updateChargeableItem(Part part, int numberOfItems)
    throws DomainException {
        try {
            ServiceData data = getServiceDM().updateChargeableItem(getData(),
            part.getData(), numberOfItems);
            reset(data);
        }
        catch (DatabaseNotAvailableException ex) {
            throw new DomainException("Not signed on.", ex);
        }
        catch (DataMgmtException ex) {
            throw new DomainException("Unable to update number of items used", ex);
        }
    }

    public void setLabourAdjustments(long labourAdjustments)
    throws DomainException {
        try {
            ServiceData data = getServiceDM().adjustServiceLabour(getData(),
            labourAdjustments);
            reset(data);
        }
        catch (DatabaseNotAvailableException ex) {
            throw new DomainException("Not signed on.", ex);
        }
        catch (DataMgmtException ex) {
            throw new DomainException("Unable to adjust labour costs", ex);
        }
    }

    public void setChargeableItemAdjustments(long chargeableItemAdjustments)
    throws DomainException {
        try {
            ServiceData data = getServiceDM().adjustServiceChargeableItems(getData(),
            chargeableItemAdjustments);
            reset(data);
        }
        catch (DatabaseNotAvailableException ex) {
            throw new DomainException("Not signed on.", ex);
        }
        catch (DataMgmtException ex) {
            throw new DomainException("Unable to adjust costs of chargeable items", ex);
        }
    }

    public void pickUpCar(java.sql.Date pickedUp) throws DomainException {
        try {
            ServiceData data = getServiceDM().pickUpCar(getData(), pickedUp);
            reset(data);
        }
        catch (DatabaseNotAvailableException ex) {
            throw new DomainException("Not signed on.", ex);
        }
        catch (DataMgmtException ex) {
            throw new DomainException("Unable to pick up car", ex);
        }
        getCar().refreshServices();
        // since RI not maintained, need to tell the car
        // its update its version of the services
    }

    //////////////////////////////////////////////////////////
    public String getLicensePlate() {
        return data.getLicensePlate();
    }

    public java.sql.Date getBookedIn() {
        return data.getBookedIn();
    }

    public int getActualMileage() {
        return data.getActualMileage();
    }

    public long getLabourAdjustments() {
        return data.getLabourAdjustments();
    }

    public long getChargeableItemAdjustments() {
        return data.getChargeableItemAdjustments();
    }

    public java.sql.Date getPickedUp() {
        return data.getPickedUp();
    }

    //////////////////////////////////////////////////////
    public List labour() throws DomainException {
        List labour = new ArrayList();
        for (Iterator iter = data.getLabour().values().iterator();
        iter.hasNext(); ) {
            ServiceData.Labour labourData = (ServiceData.Labour) iter.next();
            EmployeeData employeeData = labourData.getMechanic();
            Employee mechanic = getEmployeeFactory().getEmployee(employeeData.getId(),
            employeeData);
            labour.add(
            new Labour(mechanic, labourData.getHours(),
            labourData.getNotes()));
        }
        this.labour = labour;
        return labour;
    }

    public List chargeableItems() throws DomainException {
        List chargeableItems = new ArrayList();
        for (Iterator iter = data.getChargeableItems().values().iterator();
        iter.hasNext(); ) {
            ServiceData.ChargeableItems chargeableItemsData =
            (ServiceData.ChargeableItems) iter.next();
            PartData partData = chargeableItemsData.getPart();
            Part part = getPartFactory().getPart(partData.getNumber(), partData);
            chargeableItems.add(
            new ChargeableItem(part, chargeableItemsData.getNumberOfItems()));
        }
        this.chargeableItems = chargeableItems;
        return chargeableItems;
    }

    //////////////////////////////////////////////////////
    private ServiceDM getServiceDM() {
        return DomainSingletonManager.getInstance().getServiceDM();
    }

    private CarFactory getCarFactory() {
        return DomainSingletonManager.getInstance().getCarFactory();
    }

    private EmployeeFactory getEmployeeFactory() {
        return DomainSingletonManager.getInstance().getEmployeeFactory();
    }

    private PartFactory getPartFactory() {
        return DomainSingletonManager.getInstance().getPartFactory();
    }

    ServiceData getData() {
        return data;
    }

    public String toString() {
        return getLicensePlate() + ":" + getBookedIn() + ":" +
        getActualMileage();
    }
    //////////////////////////////////////////////////////

    /**
     * @supplierCardinality 1
     * @link aggregationByValue
     */
    private ServiceData data;

    /**
     * For appearances only; in fact, return labour on-the-fly based
     * on underlying data
     * @link aggregation
     *   @associates <{com.bettersoftwarefaster.carserv.domain.Labour}>
     * @clientCardinality 1
     * @supplierCardinality 0..*
     * @supplierRole labour
     * @constraint {ordered}
     */
    private List labour;

    /**
     * For appearances only; in fact, return chargeableItems on-the-fly based
     * on underlying data
     * @link aggregation
     *   @associates <{ChargeableItem}>
     * @clientCardinality 1
     * @supplierCardinality 0..*
     * @supplierRole chargeableItems
     * @constraint {ordered}
     */
    private List chargeableItems;

    /** @bidirectional */
    private Car car;

    private DomainSessionManager getSessionManager() {
        return DomainSingletonManager.getInstance().getDomainSessionManager();
    }
}
