package com.bettersoftwarefaster.carserv.infra;

/**
 * Represents an executable and possibly undoable command.  Nested classes
 * ExecReturn and UndoReturn define enumerated types which indicate whether
 * the command was undoable/redoable or not; these return types are used by
 * the CommandManager (via CommandInvocation) to keep track of
 * undo/redo functionality.
 * @pattern Command
 * @stereotype description
 * @see CommandManager
 * @see CommandInvocation
 */
public interface Command {
	/*
	 * @return whether this action, now executed, can be undone.
	 */

    public ExecReturn execute(Object arg) throws CommandException;

    /**
     * relies on some other agent (likely to be
     * CommandHistory/CommandInvocation) to
     * supply the argument that was originally used when this (type of)
     * command was invoked.
     * @return whether this action, now undone, can be redone.
     */
    public UndoReturn undo(Object arg) throws CommandException;

    public abstract String toString();

    public final static class ExecReturn {
        public final static ExecReturn UNDOABLE =
        new ExecReturn("Undoable", 1);
        public final static ExecReturn NOT_UNDOABLE =
        new ExecReturn("Not undoable", 2);
        public final static ExecReturn IGNORE = new ExecReturn("Ignore", 3);
        public final static ExecReturn CLEAR = new ExecReturn("Clear", 4);

        private ExecReturn(String name, int value) {
            this.name = name;
            this.value = value;
        }

        public boolean equals(Object other) {
            if (!(other instanceof ExecReturn)) {
                return false;
            }
            ExecReturn otherAsExecReturn = (ExecReturn)other;
            return (value == otherAsExecReturn.value);
        }

        public int hashCode() { return value; }

        public String toString() { return name; }

        private String name;
        private int value;
    }

    public final static class UndoReturn {
        public final static UndoReturn REDOABLE =
        new UndoReturn("Redoable", 1);
        public final static UndoReturn NOT_REDOABLE =
        new UndoReturn("Not redoable", 2);
        public final static UndoReturn CLEAR = new UndoReturn("Clear", 4);

        private UndoReturn(String name, int value) {
            this.name = name;
            this.value = value;
        }

        public boolean equals(Object other) {
            if (!(other instanceof UndoReturn)) {
                return false;
            }
            UndoReturn otherAsUndoReturn = (UndoReturn)other;
            return (value == otherAsUndoReturn.value);
        }

        public int hashCode() { return value; }

        public String toString() { return name; }

        private String name;
        private int value;
    }
}
