package com.bettersoftwarefaster.carserv.infra;

import java.util.*;

/**
 * Manages execution of undo/redo of CommandInvocations.  Can also execute
 * Commands directly, though simply instantiates a CommandInvocation to wrap
 * the Command to do this.
 */
public class CommandManager {
    public CommandManager() {
        this.undoCommandInvocations = new Stack();
        this.redoCommandInvocations = new Stack();
        this.listeners = new ArrayList();
    }

    /**
     * Execute a command invocation, and store if undoable. The redo history
     * is cleared; the user has started a new piece of work.
     */
    public void execute(CommandInvocation commandInvocation)
    throws CommandException {
        Command.ExecReturn execReturn;
        try {
            execReturn = commandInvocation.execute();
        }
        catch (CommandException ex) {
            this.mostRecentException = ex;
            throw ex;
        }
        if (execReturn == Command.ExecReturn.UNDOABLE) {
            undoCommandInvocations.push(commandInvocation);
            redoCommandInvocations.clear();
        }
        if (execReturn != Command.ExecReturn.NOT_UNDOABLE) {
            redoCommandInvocations.clear();
        }
        if (execReturn == Command.ExecReturn.CLEAR) {
            undoCommandInvocations.clear();
            redoCommandInvocations.clear();
        }
        for (Iterator iter = listeners.iterator(); iter.hasNext(); ) {
            CommandHistoryListener eachListener =
            (CommandHistoryListener)iter.next();
            eachListener.executedCommandInvocation(
            new CommandHistoryEvent(this, commandInvocation));
        }
    }

    public void execute(Command command, Object arg) throws CommandException {
        execute(new SimpleCommandInvocation(command, arg));
    }

    public void undo() throws CommandException {
        CommandInvocation commandInvocation;
        Command.UndoReturn undoReturn;
        try {
            commandInvocation = (CommandInvocation)undoCommandInvocations.pop();
            undoReturn = commandInvocation.undo();
        }
        catch (EmptyStackException ex) {
            throw new CommandException("Undo history is empty.");
        }
        catch (CommandException ex) {
            undoCommandInvocations.clear(); // no point holding if failed
            this.mostRecentException = ex;
            throw ex;
        }
        if (undoReturn == Command.UndoReturn.REDOABLE) {
            redoCommandInvocations.push(commandInvocation);
        }
        if (undoReturn == Command.UndoReturn.CLEAR) {
            undoCommandInvocations.clear();
            redoCommandInvocations.clear();
        }
        for (Iterator iter = listeners.iterator(); iter.hasNext(); ) {
            CommandHistoryListener eachListener =
            (CommandHistoryListener)iter.next();
            eachListener.undoneCommandInvocation(
            new CommandHistoryEvent(this, commandInvocation));
        }
    }

    public void redo() throws CommandException {
        CommandInvocation commandInvocation;
        try {
            commandInvocation = (CommandInvocation)redoCommandInvocations.pop();
            // no need to check if undoable; must have been since
            // ended up on the redo stack
            commandInvocation.execute();
            undoCommandInvocations.push(commandInvocation);
        }
        catch (EmptyStackException ex) {
            throw new CommandException("Redo history is empty.");
        }
        for (Iterator iter = listeners.iterator(); iter.hasNext(); ) {
            CommandHistoryListener eachListener =
            (CommandHistoryListener)iter.next();
            eachListener.redoneCommandInvocation(
            new CommandHistoryEvent(this, commandInvocation));
        }
    }

    public void clear() {
        undoCommandInvocations.clear();
        redoCommandInvocations.clear();
        for (Iterator iter = listeners.iterator(); iter.hasNext(); ) {
            CommandHistoryListener eachListener =
            (CommandHistoryListener)iter.next();
            eachListener.clearedHistory(new CommandHistoryEvent(this, null));
        }
    }

    public boolean isUndoEmpty() {
        return undoCommandInvocations.isEmpty();
    }

    public boolean isRedoEmpty() {
        return redoCommandInvocations.isEmpty();
    }

    public int getSizeOfUndoList() {
        return undoCommandInvocations.size();
    }

    public int getSizeOfRedoList() {
        return redoCommandInvocations.size();
    }

    public String toString() {
        return "CommandManager (" + System.identityHashCode(this) +
        "): numberOfListeners=" + getNumberOfListeners() +
        ", sizeOfUndoList=" + getSizeOfUndoList() + ", sizeOfRedoList=" +
        getSizeOfRedoList();
    }

    public CommandException getMostRecentException() {
        return mostRecentException;
    }

    private int getNumberOfListeners() {
        return listeners.size();
    }

    public void addCommandHistoryListener(CommandHistoryListener listener) {
        listeners.add(listener);
    }

    public void removeCommandHistoryListener(CommandHistoryListener listener) {
        listeners.remove(listener);
    }

    /**
     * @link aggregation
     *   @associates <{CommandInvocation}>
     * @clientCardinality   1
     * @supplierCardinality 0..*
     * @constraint {stack}
     * @supplierRole undo
     */
    private final Stack undoCommandInvocations;

    /**
     * @link aggregation
     *   @associates <{CommandInvocation}>
     * @clientCardinality   1
     * @supplierCardinality 0..*
     * @constraint {stack}
     * @supplierRole redo
     */
    private final Stack redoCommandInvocations;

    /**
     * @link aggregation
     *   @associates <{CommandHistoryListener}>
     * @clientCardinality   1
     * @supplierCardinality 0..*
     * @constraint {ordered}
     */
    private final List listeners;

    /** @supplierRole mostRecent */
    private CommandException mostRecentException;
}
