package com.bettersoftwarefaster.carserv.infra;

import java.util.*;

/**
 * Allows the same argument to be applied to a collection of commands.  For
 * example, the user might wish to move a group of shapes by some
 * displacement.  If there is no inbuilt composite hierarchy for the shapes
 * themselves, then separate command objects could be created for each
 * different type of shapes.  A composite command would then be created to
 * move the group of shapes as a single request.
 * <p>Do not confuse with composite command invocation, which allows different
 * arguments to be specified for each contained command invocation.
 * @pattern Composite
 * @patternRole composite
 * @stereotype description
 * @see CompositeCommandInvocation
 */
public class CompositeCommand implements Command {

    public CompositeCommand(List commands) {
        for (Iterator iter = commands.iterator(); iter.hasNext(); ) {
            try {
                Command command = (Command)iter.next();
            }
            catch (ClassCastException ex) {
                throw new IllegalArgumentException("Require list of 'Command's.");
            }
        }
        this.commands = Collections.unmodifiableList(commands);
    }

    public Command.ExecReturn execute(Object arg) throws CommandException {
        boolean allUndoable = true;
        boolean allIgnorable = true;
        for (Iterator iter = commands.iterator(); iter.hasNext(); ) {
            Command command = (Command)iter.next();
            Command.ExecReturn execReturn = command.execute(arg);
            allUndoable = (execReturn == Command.ExecReturn.UNDOABLE) &
            allUndoable;
            allIgnorable = (execReturn == Command.ExecReturn.IGNORE) &
            allIgnorable;
        }
        if (allUndoable) {
            return Command.ExecReturn.UNDOABLE;
        }
        else if (allIgnorable) {
            return Command.ExecReturn.IGNORE;
        }
        else {
            return Command.ExecReturn.NOT_UNDOABLE;
        }

    }

    /**
     * undo() in superclass will have verified that all underlying commands are
     * undoable before attempting. However, if there is a problem midway
     * through, this implementation will
     * <b>not</b> redo those successfully undone (that is, it is not
     * transactional)
     */
    public Command.UndoReturn undo(Object arg) throws CommandException {
        boolean allRedoable = true;
        for (ListIterator iter = commands.listIterator(commands.size());
        iter.hasPrevious(); ) {
            Command command = (Command)iter.previous();
            Command.UndoReturn undoReturn = command.undo(arg);
            allRedoable = (undoReturn == Command.UndoReturn.REDOABLE) &
            allRedoable;
        }
        if (allRedoable) {
            return Command.UndoReturn.REDOABLE;
        }
        else {
            return Command.UndoReturn.NOT_REDOABLE;
        }
    }

    public String toString() {
        StringBuffer buf = new StringBuffer();
        int i = 0;
        for (Iterator iter = commands.iterator(); iter.hasNext(); ) {
            Command eachCommand = (Command)iter.next();
            buf.append("\ncommands[");
            buf.append(i++);
            buf.append("] = ");
            buf.append(eachCommand.toString());
        }
        return "CompositeCommand:" + buf.toString();
    }

    /**
     * @link aggregation
     *   @associates <{Command}>
     * @clientCardinality 1
     * @supplierCardinality 0..*
     * @constraint {ordered}
     */
    private List commands;
}
