package com.bettersoftwarefaster.carserv.infra;

import java.util.*;

/**
 * Represents a collection of command invocations to be treated as a unit.
 * For example, in a WordProcessor, a user might type a number of separate
 * letters.  If the user were to press the space bar (indicating the end of a
 * word), then these separate command invocations for each letter might then
 * be replaced by a single composite command invocation that represents the
 * full word.  Similarly a bunch of word invocations could be replaced by a
 * single command invocation for a phrase.
 * <p>The composite command invocation differs from the composite command
 * because the latter relates to applying the same argument to a number of
 * commands, whereas this class allows different arguments for each command
 * invocation.  The composite command is useful to perform the same action on
 * many objects, eg moving a group of shapes.
 * @pattern Composite
 * @patternRole composite
 * @stereotype moment-interval
 * @see CompositeCommand
 */
public class CompositeCommandInvocation implements CommandInvocation {

    public CompositeCommandInvocation() {
        this(Collections.EMPTY_LIST);
    }

    public CompositeCommandInvocation(List commandInvocations) {
        for (Iterator iter = commandInvocations.iterator(); iter.hasNext(); ) {
            try {
                CommandInvocation commandInvocation =
                (CommandInvocation)iter.next();
            }
            catch (ClassCastException ex) {
                throw new IllegalArgumentException("Require list of 'CommandInvocation's.");
            }
        }
        this.commandInvocations = new ArrayList(commandInvocations);
    }

    public void add(CommandInvocation commandInvocation) {
        commandInvocations.add(commandInvocation);
    }

    public void remove(CommandInvocation commandInvocation) {
        commandInvocations.remove(commandInvocation);
    }

    public Command.ExecReturn execute() throws CommandException {
        boolean allUndoable = true;
        boolean allIgnorable = true;
        for (Iterator iter = commandInvocations.iterator(); iter.hasNext(); ) {
            CommandInvocation commandInvocation =
            (CommandInvocation)iter.next();
            Command.ExecReturn execReturn = commandInvocation.execute();
            allUndoable = (execReturn == Command.ExecReturn.UNDOABLE) &
            allUndoable;
            allIgnorable = (execReturn == Command.ExecReturn.IGNORE) &
            allIgnorable;
        }
        if (allUndoable) {
            return Command.ExecReturn.UNDOABLE;
        }
        else if (allIgnorable) {
            return Command.ExecReturn.IGNORE;
        }
        else {
            return Command.ExecReturn.NOT_UNDOABLE;
        }
    }

    /**
     * Verifies that all underlying commands are undoable before attempting.
     * However, if there is a problem midway through, will not redo those
     * successfully undone (ie is not transactional)
     * <p>Note that we traverse the invocations backwards.
     */
    public Command.UndoReturn undo() throws CommandException {
        boolean allRedoable = true;
        for (ListIterator iter =
        commandInvocations.listIterator(commandInvocations.size());
        iter.hasPrevious(); ) {
            CommandInvocation commandInvocation =
            (CommandInvocation)iter.previous();
            Command.UndoReturn undoReturn = commandInvocation.undo();
            allRedoable = (undoReturn == Command.UndoReturn.REDOABLE) &
            allRedoable;
        }
        if (allRedoable) {
            return Command.UndoReturn.REDOABLE;
        }
        else {
            return Command.UndoReturn.NOT_REDOABLE;
        }
    }

    public String toString() {
        StringBuffer buf = new StringBuffer();
        int i = 0;
        for (Iterator iter = commandInvocations.iterator(); iter.hasNext(); ) {
            CommandInvocation eachCommandInvocation =
            (CommandInvocation)iter.next();
            buf.append("\ncommandInvocations[");
            buf.append(i++);
            buf.append("] = ");
            buf.append(eachCommandInvocation.toString());
        }
        return "CompositeCommandInvocation:" + buf.toString();
    }

    /**
     * @link aggregation
     *   @associates <{CommandInvocation}>
     * @clientCardinality 1
     * @supplierCardinality 0..*
     * @constraint {ordered}
     */
    private List commandInvocations;
}

