package com.bettersoftwarefaster.carserv.infra;

import com.bettersoftwarefaster.carserv.app.*;
import com.bettersoftwarefaster.carserv.domain.*;
import java.beans.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.table.*;

/**
 * Enhanced version of JTable.  Aims to make it easier to create tables where
 * columns are of a specific type.  Only single (row) selection mode
 * is provided for.
 * <p>Use addColumn(HeadedTable.HeadedTableColumn) rather than
 * addColumn(TableColumn).  The HeadedTableColumn nested class provides the
 * required type information to the HeadedTable container.
 * <p>When adding to a panel, use getScrollPane() for an automatically
 * provided wrapper.
 * <p>Implementation notes: unifies the DefaultTableModel,
 * DefaultTableColumnModel (with ListSelectionModel defined by the
 * DefaultTableColumnModel), and also independently keeps track of columns.
 */
public class HeadedTable extends JTable {
    public HeadedTable() {
        super(new DefaultTableModel(), new DefaultTableColumnModel());
        getSelectionModel().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        scrollPane = new JScrollPane();
        scrollPane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        scrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
        scrollPane.setViewportView(this);
        columnList = new ArrayList(10);
    }

    public JScrollPane getScrollPane() {
        return scrollPane;
    }

    private JScrollPane scrollPane;

    public void addListSelectionListener(ListSelectionListener l) {
        ListSelectionModel rowSM = getSelectionModel();
        rowSM.addListSelectionListener(l);
    }

    public void addColumn(HeadedTableColumn headedTableColumn) {
        getDefaultTableModel().addColumn(headedTableColumn.getHeaderValue());
        if (!headedTableColumn.isHidden()) {
            getDefaultTableColumnModel().addColumn(headedTableColumn);
        }
        columnList.add(headedTableColumn);
    }

    public boolean isCellEditable(int row, int viewedColumn) {
        TableColumn tableColumn = getColumnModel().getColumn(viewedColumn);
        if (tableColumn instanceof HeadedTableColumn) {
            return ((HeadedTableColumn)tableColumn).isEditable();
        }
        else {
            return false;
        }
    }

    public Class getColumnClass(int viewedColumn) {
        TableColumn tableColumn = getColumnModel().getColumn(viewedColumn);
        if (tableColumn instanceof HeadedTableColumn) {
            HeadedTableColumn headedTableColumn =
            (HeadedTableColumn)tableColumn;
            if (headedTableColumn.getColumnClass() != null) {
                return headedTableColumn.getColumnClass();
            }
        }
        return super.getColumnClass(viewedColumn);
    }

    public void clear() {
        for (int i = getDefaultTableModel().getRowCount(); i > 0; i--) {
            getDefaultTableModel().removeRow(i - 1);
        }
    }

/*    public void setValueAt(Object value, int row, int col) {
		getDefaultTableModel().setValueAt(value, row, col);
	}
*/

    public DefaultTableModel getDefaultTableModel() {
        return (DefaultTableModel)getModel();
    }

    public DefaultTableColumnModel getDefaultTableColumnModel() {
        return (DefaultTableColumnModel)getColumnModel();
    }

    public void setEnabled(boolean enabled) {
        super.setEnabled(enabled);
        this.enabled = enabled;
    }

    public boolean getEnabled() {
        return this.enabled;
    }

    private boolean enabled;
    private java.util.List columnList;

    public abstract class SelectionListener implements ListSelectionListener {
        public final void valueChanged(ListSelectionEvent ev) {
            if (ev.getValueIsAdjusting()) {
                selectedRow = -1; // none
                return;
            }
            ListSelectionModel lsm = (ListSelectionModel)ev.getSource();
            if (lsm.isSelectionEmpty()) {
                selectedRow = -1; // none
                return;
            }
            else {
                selectedRow = lsm.getMinSelectionIndex();
            }
            doValueChanged(ev);
        }

        private int selectedRow;

        public int getSelectedRow() {
            return selectedRow;
        }

        public Object getHiddenValue() {
            if (getSelectedRow() == -1) {
                return null;
            }
            for (int i = 0; i < columnList.size(); i++) {
                HeadedTableColumn headedTableColumn =
                (HeadedTableColumn)columnList.get(i);
                if (headedTableColumn.isHidden()) {
                    return getDefaultTableModel().getValueAt(getSelectedRow(), i);
                }
            }
            return null;
        }

        public abstract void doValueChanged(ListSelectionEvent ev);
    }

    public static class HeadedTableColumn extends TableColumn {
        public HeadedTableColumn(int modelIndex, int preferredWidth,
        String headerValue, boolean editable) {
            super(modelIndex, preferredWidth);
            this.headerValue = headerValue;
            this.editable = editable;
            this.hidden = false;
        }

        public HeadedTableColumn(int modelIndex, int preferredWidth,
        String headerValue, boolean editable, Class columnClass) {
            this(modelIndex, preferredWidth, headerValue, editable);
            this.columnClass = columnClass;
        }

        public HeadedTableColumn(int modelIndex, int preferredWidth,
        String headerValue, boolean editable, TableCellRenderer cellRenderer) {
            this(modelIndex, preferredWidth, headerValue, editable);
            setCellRenderer(cellRenderer);
        }

        public HeadedTableColumn(int modelIndex, int preferredWidth,
        String headerValue, TableCellEditor cellEditor) {
            this(modelIndex, preferredWidth, headerValue, true);
            setCellEditor(cellEditor);
        }

        public HeadedTableColumn(int modelIndex, int preferredWidth,
        String headerValue, TableCellEditor cellEditor, Class columnClass) {
            this(modelIndex, preferredWidth, headerValue, true);
            setCellEditor(cellEditor);
            this.columnClass = columnClass;
        }

        public HeadedTableColumn(int modelIndex, int preferredWidth,
        String headerValue, TableCellRenderer cellRenderer,
        TableCellEditor cellEditor) {
            this(modelIndex, preferredWidth, headerValue, true);
            setCellRenderer(cellRenderer);
            setCellEditor(cellEditor);
        }

        /** Constructor for hidden columns */
        public HeadedTableColumn(int modelIndex, String headerValue) {
            super(modelIndex);
            this.headerValue = headerValue;
            this.editable = false;
            this.hidden = true;
        }

        public Object getHeaderValue() {
            return headerValue;
        }

        public boolean isEditable() {
            return editable;
        }

        public boolean isHidden() {
            return hidden;
        }

        public Class getColumnClass() {
            return columnClass;
        }

        private Class columnClass;
        private final String headerValue;
        private final boolean editable;
        private final boolean hidden;
    }
}
