package com.bettersoftwarefaster.carserv.infra;

import java.util.*;

/**
 * Register of singletons, with each singleton held in a Map by name.  Hence,
 * acts a little like a resource manager, something like JNDI.
 * <p>This design uses a cobweb-like effect; all singletons are registered in
 * the SingletonRegister, and then an instance to SingletonRegister is held
 * somewhere (typically, this would be a class representing the
 * logical application).
 * <P>An alternative design for implementing singletons is to have each class
 * keep a reference to themselves.  This is okay provided that the JVM does
 * not do class unloading.
 */
public class SingletonRegister {
    public SingletonRegister() {
    }

    /**
     * @returns previous object registered as singleton with that name, if any
     */
    public Object bind(String name, Object singleton) {
        return singletons.put(name, singleton);
    }

    /** @returns old singleton registered with that name, if any */
    public Object unbind(String name) {
        return singletons.remove(name);
    }

    public Object get(String name) {
        Object singleton = singletons.get(name);
        if (singleton == null) {
            throw new IllegalArgumentException("No singleton named <" +
            name + ">");
        }
        return singleton;
    }

    /**
     * @link aggregation
     *   @associates <{Object}>
     *   @clientQualifier name
     * @clientCardinality 1
     * @supplierCardinality 0..*
     * @supplierRole singletons
     * @constraint {set}
     */
    protected Map singletons = new HashMap();
}
