package com.bettersoftwarefaster.carserv.infra;

import java.util.*;

/**
 * Decorates a CommandInvocation with a logging command.  Hence, can simply
 * trace which command invocationss are being executed/undone.
 * <p>If no Logger is supplied, then a PrintLogger is used.
 * <p>The difference between this and TraceCommand is that this relates only
 * to a single invocation of the underlying command, whereas a TraceCommand
 * will always trace the underlying command, no matter what invokes it.
 * Hence, only ever need to enable tracing at the command invocation or at
 * the command layer, not at both. <p>Implementation note: instantiates a
 * TraceCommand to actually perform the tracing.
 * @stereotype moment-interval
 * @see PrintLogger
 * @see TraceCommand
 */
public class TraceCommandInvocation extends LeafCommandInvocation {
    public TraceCommandInvocation(CommandInvocation commandInvocation) {
        this(commandInvocation, new PrintLogger());
    }

    public TraceCommandInvocation(CommandInvocation commandInvocation,
    Logger logger) {
        this.commandInvocation = commandInvocation;
        this.logger = logger;
    }

    public Command.ExecReturn execute() throws CommandException {
        getLogger().log("executing: " + getCommandInvocation());
        return getCommandInvocation().execute();
    }

    public Command.UndoReturn undo() throws CommandException {
        getLogger().log("undoing: " + getCommandInvocation());
        return getCommandInvocation().undo();
    }

    public CommandInvocation getCommandInvocation()
    { return commandInvocation; }

    public Logger getLogger() { return logger; }

    public String toString() {
        return "TraceCommandInvocation; logger = " + logger +
        ", commandInvocation = " + getCommandInvocation();
    }

    /**
     * @clientCardinality 1
     * @supplierCardinality 1
     */
    final private CommandInvocation commandInvocation;

    /**
     * @clientCardinality 1
     * @supplierCardinality 1
     */
    final private Logger logger;
}
