package com.bettersoftwarefaster.carserv.ui;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import com.bettersoftwarefaster.carserv.infra.*;
import com.bettersoftwarefaster.carserv.app.*;
import com.bettersoftwarefaster.carserv.domain.*;

/**
 * The entry point into the application.  Creates the UI and a reference to
 * app.CarServApp.
 */
public class CarServAppUI extends JFrame {
    /**
     * Access to CarServAppUI, itself a Singleton.
     * This saves us having to pass the thing around to the constructor of
     * every class in this package.
     */
    public static CarServAppUI getInstance() {
        if (instance == null) {
            throw new IllegalStateException("Not yet initialised.");
        }
        return instance;
    }

    /** @supplierRole instance */
    private static CarServAppUI instance;

    public static void main(String[] args) throws UiException {
        instance = new CarServAppUI(args);
        instance.centreOnScreen();
        instance.show();
    }

    private CarServAppUI(String[] args) throws UiException {
        this.args = args;
        try {
            this.carServApp = new CarServApp();
        }
        catch (AppException ex) {
            throw new UiException("Unable to initialise application package", ex);
        }
        this.commandManager = carServApp.getAppSingletonManager().getCommandManager();
        buildGui();
        bindSingletons();
    }

    private void buildGui() {
        setTitle("Car Servicing System");
        addMenuBar();
        tabbedPane = new MappedTabbedPane();
        tabbedPane.add(createDiaryTabPanel(), "Diary");
        tabbedPane.add(createServiceTabPanel(), "Service");
        currentUserPanel = new CurrentUserPanel();
        getContentPane().add("North", currentUserPanel);
        getContentPane().add("Center", tabbedPane);
        this.statusBar = new StatusBar();
        getContentPane().add("South", thinBordered(statusBar));
        pack();
        addSignOnListener(currentUserPanel);
        setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        addWindowListener(
        new WindowAdapter() {
            public void windowClosing(WindowEvent ev) {
                quitApplicationAfterConfirm();
            }
        });
        userLoggedOut();

        /**
         * testing only, as a convenience
         *		 SignOnCommand signOnCommand = new SignOnCommand() {
         *            public Object getArg(ActionEvent ev) {
         *                return new SignOnCommand.Arg("Jacques", "Hattie");
         *            }
         *            public void postUndo(Object arg) {
         *                userLoggedOut();
         *            }
         *            public void postExecute(Object arg) {
         *                if (this.getApp().getCurrentUser() == null) {
         *                    // shouldn't happen; call executeFailed() instead
         *                    return;
         *                }
         *                userSignedOn();
         *		    }
         *		    public void executeFailed(CommandException ex) {
         *		        getStatusBar().setMessage(ex);
         *		     }
         *		};
         *		signOnCommand.executeSelf();
         */
    }

    private void addMenuBar() {
        JMenuBar menuBar = new JMenuBar();
        menuBar.add(createFileMenu());
        menuBar.add(createEditMenu());
        menuBar.add(createViewMenu());
        setJMenuBar(menuBar);
    }

    private JMenu createFileMenu() {
        JMenu fileMenu = new MnemonicMenu("File", 'F');
        final JMenuItem fileSignOnMenuItem = fileMenu.add(
        new AppAction("Sign on", 'S') {
            public void actionPerformed(ActionEvent ev) {
                new SignOnDialog().show();
            }
        });
        final JMenuItem fileLogoutMenuItem = fileMenu.add(
        new LogoutCommand('L') {
            public Object getArg(ActionEvent ev) {
                int areYouSureOption =
                JOptionPane.showConfirmDialog(CarServAppUI.this,
                "Are you sure you want to logout?", CarServAppUI.this.getTitle(),
                JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
                if (areYouSureOption == JOptionPane.YES_OPTION) {
                    return new LogoutCommand.Arg(getCurrentUser());
                }
                else {
                    return null;
                }
            }
            public void postExecute(Object arg) {
                getStatusBar().setMessage("Logged out.");
                notifyListenersLoggedOut();
            }
            public void executeFailed(CommandException ex) {
                getStatusBar().setMessage(ex);
            }
        });
        this.addSignOnListener(
        new SignOnListener() {
            public void userSignedOn(SignOnEvent ev) {
                fileSignOnMenuItem.setEnabled(false);
                fileLogoutMenuItem.setEnabled(true);
            }
            public void userLoggedOut(SignOnEvent ev) {
                fileSignOnMenuItem.setEnabled(true);
                fileLogoutMenuItem.setEnabled(false);
            }
        });
        fileMenu.add(
        new AppAction("Exit", 'x') {
            public void actionPerformed(ActionEvent ev) {
                quitApplicationAfterConfirm();
            }
        });
        return fileMenu;
    }

    private JMenu createEditMenu() {
        JMenu editMenu = new MnemonicMenu("Edit", 'E');
        final JMenuItem editUndoMenuItem = editMenu.add(
        new AppAction("Undo", 'U') {
            public void actionPerformed(ActionEvent ev) {
                try {
                    commandManager.undo();
                }
                catch (CommandException ex) {
                    System.err.println("Problem undoing command, ex = " + ex);
                }
            }
        });
        final JMenuItem editRedoMenuItem = editMenu.add(
        new AppAction("Redo", 'R') {
            public void actionPerformed(ActionEvent ev) {
                try {
                    commandManager.redo();
                }
                catch (CommandException ex) {
                    System.err.println("Problem redoing command, ex = " + ex);
                }
            }
        });
        this.commandManager.addCommandHistoryListener(
        new CommandHistoryListener() {
            public void undoneCommandInvocation(CommandHistoryEvent ev) {
                setEnabledMenuItems();
            }
            public void redoneCommandInvocation(CommandHistoryEvent ev) {
                setEnabledMenuItems();
            }
            public void executedCommandInvocation(CommandHistoryEvent ev) {
                setEnabledMenuItems();
            }
            public void clearedHistory(CommandHistoryEvent ev) {
                setEnabledMenuItems();
            }
            private void setEnabledMenuItems() {
                editUndoMenuItem.setEnabled(!commandManager.isUndoEmpty());
                editRedoMenuItem.setEnabled(!commandManager.isRedoEmpty());
            }
        });
        editUndoMenuItem.setEnabled(false);
        editRedoMenuItem.setEnabled(false);
        return editMenu;
    }

    private JMenu createViewMenu() {
        JMenu viewMenu = new MnemonicMenu("View", 'V');
        viewMenu.add(
        new AppAction("Diary", 'D') {
            public void actionPerformed(ActionEvent ev) {
                tabbedPane.view(this.getName());
            }
        });
        viewMenu.add(
        new AppAction("Service", 'S') {
            public void actionPerformed(ActionEvent ev) {
                tabbedPane.view(this.getName());
            }
        });
        return viewMenu;
    }

    private JComponent bordered(JComponent component) {
        return bordered(component, 5);
    }

    private JComponent thinBordered(JComponent component) {
        Border border = new CompoundBorder(new EmptyBorder(5, 0, 0, 5),
        new SoftBevelBorder(BevelBorder.LOWERED));
        component.setBorder(border);
        return component;
    }

    private JComponent bordered(JComponent component, int margin) {
        Border border = new CompoundBorder(
        new CompoundBorder(new EmptyBorder(margin, margin, margin, margin),
        new SoftBevelBorder(BevelBorder.LOWERED)),
        new EmptyBorder(margin, margin, margin, margin));
        component.setBorder(border);
        return component;
    }

    private JPanel createDiaryTabPanel() {
        JPanel diaryTabPanel = new JPanel(new BorderLayout());
        final DiaryPanel diaryPanel = new DiaryPanel('t', 'x');
        CustomerCarPanel customerCarPanel = new CustomerCarPanel('k', 'C');
        CarServicePanel carServicePanel = new CarServicePanel('i');
        ScheduleServicePanel scheduleServicePanel =
        new ScheduleServicePanel('r', 'a', 'M', 'S');
        final JPanel customerCarServicePanel = new JPanel(new BorderLayout());
        customerCarServicePanel.add(bordered(customerCarPanel),
        BorderLayout.NORTH);
        customerCarServicePanel.add(bordered(carServicePanel),
        BorderLayout.CENTER);
        customerCarServicePanel.add(bordered(scheduleServicePanel),
        BorderLayout.SOUTH);
        diaryTabPanel.add(bordered(diaryPanel), BorderLayout.CENTER);
        diaryTabPanel.add(customerCarServicePanel, BorderLayout.EAST);
        customerCarPanel.addCarSelectionListener(carServicePanel); // to display services
        customerCarPanel.addCarSelectionListener(scheduleServicePanel); // to
        // schedule service for car
        diaryPanel.addDateSelectionListener(scheduleServicePanel); // to
        // schedule service for shown date
        // if schedule new service, impact diary panel and car service panel
        scheduleServicePanel.addCarServiceListener(diaryPanel); // after schedule new service
        scheduleServicePanel.addCarServiceListener(carServicePanel); // after schedule new service
        // if book in or cancel service, impact car service panel
        // (diaryPanel already listens to own changes)
        diaryPanel.addCarServiceListener(carServicePanel); // after cancel a
        // service, in case currently being displayed
        addSignOnListener(diaryPanel);
        addSignOnListener(customerCarPanel);
        addSignOnListener(carServicePanel);
        addSignOnListener(scheduleServicePanel);
        return diaryTabPanel;
    }

    private JPanel createServiceTabPanel() {
        JPanel serviceTabPanel = new JPanel();
        serviceTabPanel.setLayout(new BorderLayout());
        final JPanel customerCarServicePanel = new JPanel(new BorderLayout());
        CustomerCarPanel customerCarPanel = new CustomerCarPanel('k', 'C');
        CarServicePanel carServicePanel = new CarServicePanel('i');
        customerCarServicePanel.add(bordered(customerCarPanel),
        BorderLayout.NORTH);
        customerCarServicePanel.add(bordered(carServicePanel),
        BorderLayout.CENTER);
        final JPanel serviceDetailPanel = new JPanel(new BorderLayout());
        final ServiceHeaderPanel serviceHeaderPanel = new ServiceHeaderPanel();
        final JPanel labourPartsPanel = new JPanel(new GridLayout(1, 2));
        final TotalsPanel totalsPanel = new TotalsPanel('P');
        ServiceLabourPanel labourPanel = new ServiceLabourPanel('d');
        ServicePartsPanel partsPanel = new ServicePartsPanel('j');
        labourPartsPanel.add(labourPanel);
        labourPartsPanel.add(partsPanel);
        serviceDetailPanel.add(bordered(serviceHeaderPanel),
        BorderLayout.NORTH);
        serviceDetailPanel.add(bordered(labourPartsPanel),
        BorderLayout.CENTER);
        serviceDetailPanel.add(bordered(totalsPanel), BorderLayout.SOUTH);
        serviceTabPanel.add(customerCarServicePanel, BorderLayout.WEST);
        serviceTabPanel.add(serviceDetailPanel, BorderLayout.CENTER);
        addSignOnListener(customerCarPanel);
        addSignOnListener(carServicePanel);
        addSignOnListener(labourPanel);
        addSignOnListener(partsPanel);
        customerCarPanel.addCarSelectionListener(carServicePanel);
        carServicePanel.addServiceSelectionListener(serviceHeaderPanel);
        carServicePanel.addServiceSelectionListener(labourPanel);
        carServicePanel.addServiceSelectionListener(partsPanel);
        labourPanel.addSubTotalListener(totalsPanel);
        partsPanel.addSubTotalListener(totalsPanel);
        return serviceTabPanel;
    }

    private void quitApplicationAfterConfirm() {
        int areYouSureOption = JOptionPane.showConfirmDialog(CarServAppUI.this,
        "Are you sure you want to exit?", getTitle(),
        JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE);
        if (areYouSureOption == JOptionPane.YES_OPTION) {
            System.exit(0);
        }
    }

    private void centreOnScreen() {
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        setLocation((screenSize.width - getPreferredSize().width) / 2,
        (screenSize.height - getPreferredSize().height) / 2);
    }

    private void bindSingletons() {
        getApp().getSingletonRegister().bind("StatusBar", statusBar);
    }

    /**
     * Implementation note: returns local variable, but
     * equally could have obtained from the SingletonRegister
     */
    StatusBar getStatusBar() { return statusBar; }

    private final String[] args;

    /** callback for SignOnDialog */
    void userSignedOn() {
        getStatusBar().setMessage("Signed on.");
        notifyListenersSignedOn();
    }

    /** callback for SignOnDialog */
    void userLoggedOut() {
        notifyListenersLoggedOut();
    }

    Employee getCurrentUser() {
        return getApp().getCurrentUser();
    }

    private CarServApp getApp() {
        return carServApp;
    }

    /**
     * @link aggregationByValue
     * @supplierCardinality 1
     */
    private MappedTabbedPane tabbedPane;

    /**
     * @link aggregationByValue
     * @supplierCardinality 1
     */
    private CurrentUserPanel currentUserPanel;

    /**
     * @clientCardinality 1
     * @supplierCardinality 1
     * @link aggregationByValue
     * @label creates
     */
    private final CarServApp carServApp;

    /**
     * @supplierCardinality 1
     * @label creates
     * @link aggregationByValue
     */
    private StatusBar statusBar;

    /**
     * @link aggregationByValue
     * @supplierCardinality 1
     * @label uses
     */
    private final CommandManager commandManager;

    public void addSignOnListener(SignOnListener sol) {
        listeners.add(sol);
    }

    public void removeSignOnListener(SignOnListener sol) {
        listeners.remove(sol);
    }

    private void notifyListenersSignedOn() {
        for (Iterator iter = listeners.iterator(); iter.hasNext(); ) {
            ((SignOnListener)iter.next()).userSignedOn(
            new SignOnEvent(this, getApp().getCurrentUser()));
        }
    }

    private void notifyListenersLoggedOut() {
        for (Iterator iter = listeners.iterator(); iter.hasNext(); ) {
            ((SignOnListener)iter.next()).userLoggedOut(
            new SignOnEvent(this, null));
        }
    }

    private java.util.List listeners = new LinkedList();
}
