package test.com.bettersoftwarefaster.carserv.infra;

import junit.framework.*;
import junitx.framework.*;
//import junit.util.*;
import java.util.*;
import java.io.*;
import com.bettersoftwarefaster.carserv.infra.*;


/**
 * The purpose of this test is not to test the test.NotUndoableCommand per se,
 * but rather the template class (SingleLevelUndoCommandAdapter) from which it
 * inherits most of its functionality.
 *
 * @stereotype test
 */
public class TestCommandManager extends ActionHistoryTestCase {

	/**
	 * constructor.
	 * @param    aName     a test name
	 */
	public TestCommandManager(String aName) {
		super(aName);
	}

	public void testAddCommandHistoryListener() throws CommandException, TestAccessException {

		commandManager.addCommandHistoryListener(
			new ActionHistoryCommandHistoryListener(getActionHistory()));
		commandManager.addCommandHistoryListener(
			new CommandHistoryAdapter() );

		assertEquals(2, getNumberOfListeners());
	}

	public void testRemoveCommandHistoryListener() throws CommandException, TestAccessException {

		CommandHistoryListener listener = new ActionHistoryCommandHistoryListener(getActionHistory());
		commandManager.addCommandHistoryListener(listener);
		commandManager.addCommandHistoryListener(
			new CommandHistoryAdapter() );

		commandManager.removeCommandHistoryListener(listener);

		assertEquals(1, getNumberOfListeners());
	}

	public void testRemoveCommandHistoryListenerNoneExistent() throws CommandException, TestAccessException {

		commandManager.addCommandHistoryListener(
			new CommandHistoryAdapter() );

		// not a listener
		commandManager.removeCommandHistoryListener(
			new ActionHistoryCommandHistoryListener(getActionHistory()));

		assertEquals(1, getNumberOfListeners());
	}


	public void testExecute() throws CommandException, TestAccessException {
		commandManager.addCommandHistoryListener(
			new ActionHistoryCommandHistoryListener(getInvocationHistory()));

		/*Command.ExecReturn execReturn =*/
		commandManager.execute(commandInvocationU1);

//		assertTrue("return value", (execReturn == Command.ExecReturn.UNDOABLE) );

		assertEquals("undo list size", 1, getSizeOfUndoList());
		assertEquals("redo list size", 0, getSizeOfRedoList());
		assertNull("most recent exception", commandManager.getMostRecentException());

		ActionHistory expectedActionHistory =
			new ActionHistory(new Object[] {
			"invocation U1",
			} );
		assertEquals( "action history", expectedActionHistory, getActionHistory() );

		ActionHistory expectedInvocationHistory =
			new ActionHistory(new Object[] {
			"executed: CommandInvocation: command=<UndoableActionCommand> arg=<invocation U1>"
			} );
		assertEquals( "invocation history", expectedInvocationHistory, getInvocationHistory() );
	}

	/**
	 * use the actionHistory instead of invocationHistory,
	 * but only so that we can check the ordering
	 */
	public void testExecuteRegisterAtAnyTime() throws CommandException, TestAccessException {

		CommandHistoryListener listener =
			new ActionHistoryCommandHistoryListener(getActionHistory());

		commandManager.execute(commandInvocationU1);
		commandManager.addCommandHistoryListener(listener);
		commandManager.execute(commandInvocationU2);
		commandManager.removeCommandHistoryListener(listener);
		commandManager.execute(commandInvocationNU1);

		ActionHistory expectedActionHistory =
		new ActionHistory(new Object[] {
		// not logged
		"invocation U1",
		// should be logged
		"invocation U2",
		"executed: CommandInvocation: command=<UndoableActionCommand> arg=<invocation U2>",
		// not logged
		"invocation NU1",
		} );
		assertEquals( "action history", expectedActionHistory, getActionHistory() );
	}

	public void testExecuteThrowsException() {
		/*Command.ExecReturn execReturn = */
		try {
			commandManager.execute(
			new CommandAdapter() {
			public Command.ExecReturn execute(Object arg) throws CommandException {
				throw new CommandException("throw ex 1");
			}
			}, "invocation 1");
			fail("No exception caught, expected CommandException");
		} catch(CommandException ex) {
			// expected
		}

//		assertTrue("return value", (execReturn == Command.ExecReturn.IGNORE) );
		assertTrue("most recent exception",
			commandManager.getMostRecentException().getMessage().equals("throw ex 1"));
	}

	public void testUndo() throws CommandException, TestAccessException {

		commandManager.addCommandHistoryListener(
			new ActionHistoryCommandHistoryListener(getInvocationHistory()));

		commandManager.execute(commandInvocationU1);
		commandManager.undo();

		assertEquals("undo list size", 0, getSizeOfUndoList());
		assertEquals("redo list size", 1, getSizeOfRedoList());

		ActionHistory expectedActionHistory =
		new ActionHistory(new Object[] {
		} );
		assertEquals( "action history", expectedActionHistory, getActionHistory() );

		ActionHistory expectedInvocationHistory =
		new ActionHistory(new Object[] {
		"executed: CommandInvocation: command=<UndoableActionCommand> arg=<invocation U1>",
		"undone: CommandInvocation: command=<UndoableActionCommand> arg=<invocation U1>",
		} );
		assertEquals( "invocation history", expectedInvocationHistory, getInvocationHistory() );
	}

	public void testRedo() throws CommandException, TestAccessException {

		commandManager.addCommandHistoryListener(
			new ActionHistoryCommandHistoryListener(getInvocationHistory()));

		commandManager.execute(commandInvocationU1);
		commandManager.undo();
		commandManager.redo();

		assertEquals("undo list size", 1, getSizeOfUndoList());
		assertEquals("redo list size", 0, getSizeOfRedoList());

		ActionHistory expectedActionHistory =
		new ActionHistory(new Object[] {
		"invocation U1",
		} );
		assertEquals( "action history", expectedActionHistory, getActionHistory() );

		ActionHistory expectedInvocationHistory =
		new ActionHistory(new Object[] {
		"executed: CommandInvocation: command=<UndoableActionCommand> arg=<invocation U1>",
		"undone: CommandInvocation: command=<UndoableActionCommand> arg=<invocation U1>",
		"redone: CommandInvocation: command=<UndoableActionCommand> arg=<invocation U1>",
		} );
		assertEquals( "invocation history", expectedInvocationHistory, getInvocationHistory() );
	}

	public void testExecuteNotUndoable() throws CommandException, TestAccessException {

		commandManager.execute(commandInvocationNU1);
		assertEquals("undo list size", 0, getSizeOfUndoList());
		assertEquals("redo list size", 0, getSizeOfRedoList());
	}

	public void testUndoNotUndoable() throws CommandException, TestAccessException {

		commandManager.execute(commandInvocationNU1);

		assertEquals("undo list size", 0, getSizeOfUndoList());
		assertEquals("redo list size", 0, getSizeOfRedoList());

		ActionHistory expectedActionHistory =
		new ActionHistory(new Object[] {
		"invocation NU1",
		} );
		assertEquals( "action history", expectedActionHistory, getActionHistory() );
	}

	public void testUndoNotRedoable() throws CommandException, TestAccessException {

		commandManager.execute(commandInvocationUNR1);

		assertEquals("undo list size after execute", 1, getSizeOfUndoList());
		assertEquals("redo list size after execute", 0, getSizeOfRedoList());

		commandManager.undo();

		assertEquals("undo list size after undo", 0, getSizeOfUndoList());
		assertEquals("redo list size after undo", 0, getSizeOfRedoList());

		ActionHistory expectedActionHistory =
		new ActionHistory(new Object[] { } );
		assertEquals( "action history", expectedActionHistory, getActionHistory() );
	}

	public void testUndoEmptyUndoList() {
		try {
			commandManager.undo();
			fail("Expected to catch CommandException, instead nothing.");
		} catch(CommandException ex) {
			// expected
		}
	}

	public void testRedoEmptyRedoList() {
		try {
			commandManager.redo();
			fail("Expected to catch CommandException, instead nothing.");
		} catch(CommandException ex) {
			// expected
		}
	}

	public void testClear() throws CommandException, TestAccessException {

		commandManager.addCommandHistoryListener(
			new ActionHistoryCommandHistoryListener(getInvocationHistory()));

		commandManager.execute(commandInvocationU1);
		commandManager.execute(commandInvocationU2);
		commandManager.undo();
		commandManager.clear();

		assertEquals("undo list size", 0, getSizeOfUndoList());
		assertEquals("redo list size", 0, getSizeOfRedoList());
	}

	public void testGetMostRecentExceptionNoPrevious() {
		assertNull(commandManager.getMostRecentException());
	}

	private int getNumberOfListeners() throws TestAccessException {
		return ( (Integer)invoke(commandManager, "getNumberOfListeners", new Object[] { } ) ).intValue();
	}

	private int getSizeOfUndoList() throws TestAccessException {
		return ( (Integer)invoke(commandManager, "getSizeOfUndoList", new Object[] { } ) ).intValue();
	}

	private int getSizeOfRedoList() throws TestAccessException {
		return ( (Integer)invoke(commandManager, "getSizeOfRedoList", new Object[] { } ) ).intValue();
	}

	protected void setUp() {
		super.setUp();
		commandManager = new CommandManager();
		invocationHistory = new ActionHistory();
		commandInvocationU1 = new SimpleCommandInvocation(
			new UndoableActionCommand(getActionHistory()), "invocation U1");
		commandInvocationU2 = new SimpleCommandInvocation(
			new UndoableActionCommand(getActionHistory()), "invocation U2");
		commandInvocationNU1 = new SimpleCommandInvocation(
			new NotUndoableActionCommand(getActionHistory()), "invocation NU1");
		commandInvocationNU2 = new SimpleCommandInvocation(
			new NotUndoableActionCommand(getActionHistory()), "invocation NU2");
		commandInvocationUNR1 = new SimpleCommandInvocation(
			new NotRedoableUndoableActionCommand(getActionHistory()), "invocation UNR1");
	}

	protected void tearDown() {
		commandInvocationU1 = null;
		commandInvocationU2 = null;
		commandInvocationNU1 = null;
		commandInvocationNU2 = null;
		commandInvocationUNR1 = null;
		invocationHistory = null;
		commandManager = null;
		super.tearDown();
	}

	private CommandManager commandManager;

	private CommandInvocation commandInvocationU1;
	private CommandInvocation commandInvocationU2;
	private CommandInvocation commandInvocationNU1;
	private CommandInvocation commandInvocationNU2;

	private CommandInvocation commandInvocationUNR1;

	private ActionHistory invocationHistory;

	private ActionHistory getInvocationHistory() {
		return invocationHistory;
	}

	/**
	 * returns a <code>TestSuite</code> instance which contains this
	 * class as only test.
	 * @return  a <code>TestSuite</code>
	 */
	static public Test suite() {
		return new TestSuite(TestCommandManager.class);
	}
}
