package com.togethersoft.modules.bettersoftwarefaster.classlinks;

import com.togethersoft.modules.bettersoftwarefaster.rwisupport.*;
import com.togethersoft.openapi.ide.*;
import com.togethersoft.openapi.ide.message.*;
import com.togethersoft.openapi.rwi.*;
import com.togethersoft.openapi.rwi.enum.*;
import java.util.*;

/**
 * Iterates over the RWI model twice; first time removing any hyperlinks
 * from classes to class diagrams, and then adding them back in such that
 * each class is hyperlinked to every class diagram in which it appears.
 */
public class ClassLinksModule extends RwiVisitorSupport implements IdeScript {
    public ClassLinksModule() {
        super("Class Diagram Hyperlinks");
        state = State.COLLECTING_REFERENCES;
    }

    /**
     * @link aggregation
     * @supplierCardinality 1
     */
    private State state;

    public void run(IdeContext ideContext) {
        getMessagePage().printMessage("Started.");
        getMessagePage().setVisible(true);
        if (!isActiveProject())
            return;
        do {
            getMessagePage().printMessage(state.toString());
            getRwiVisitorDispatcher().visitAndDispatch();
            state = state.getNext();
        } while (state != State.DONE);
        getMessagePage().printMessage("Finished.");
    }

    public Object visitPackage(RwiPackage rwiPackage) {
        return null;
    }

    public boolean visitDiagrams() {
        return state == State.COLLECTING_REFERENCES;
    }

    /**
    * For any nodes that are classes or interfaces, and are imported (that is,
    * not appearing as primary references in their package diagram), add to
    * the list of diagrams mapped to that node.  The diagramList will be
    * created automatically if it does not exist.
    */
    public Object visitDiagram(RwiDiagram rwiDiagram) {
        for (RwiNodeReferenceEnumeration enum = rwiDiagram.nodeReferences(); enum.hasMoreElements(); ) {
			RwiNodeReference rwiNodeReference = enum.nextRwiNodeReference();

            if (rwiNodeReference.isImported()) {
            RwiNode rwiNode = rwiNodeReference.getNode();
            if (RwiSupport.isRwiNodeClassOrInterface(rwiNode)) {
                List diagramList = (List)rwiNodeDiagramListMap.get(rwiNode);
                if (diagramList == null) {
                    diagramList = new LinkedList();
                    rwiNodeDiagramListMap.put(rwiNode, diagramList);
                }
                diagramList.add(rwiDiagram);
            }
            }
        }
        return null;
    }

    public boolean visitNodes() {
        return (state == State.ADDING_HYPERLINKS);
    }

    public Object visitNode(RwiNode rwiNode) {
        List diagramList = (List)rwiNodeDiagramListMap.get(rwiNode);
		int addedHyperlinks = 0;
        if (diagramList == null) {
            return null;
        }
        for (Iterator iter = diagramList.iterator(); iter.hasNext(); ) {
            RwiDiagram rwiDiagram = (RwiDiagram)iter.next();

			if (!hasHyperlinkTo(rwiNode, rwiDiagram)) {
				if (rwiNode.canAddProperty(RwiProperty.HYPERLINK, RwiSupport.uniqueNameOf(rwiDiagram))) {
		            rwiNode.addProperty(RwiProperty.HYPERLINK, RwiSupport.uniqueNameOf(rwiDiagram));
	                addedHyperlinks++;
	            } else {
	                getMessagePage().printMessage(RwiSupport.fullNameOf(rwiNode) + ": unable to add hyperlink to \"" + RwiSupport.nameOf(rwiDiagram) + "\"", rwiNode);
	            }
			}
        }
		if (addedHyperlinks>0) {
            getMessagePage().printMessage(RwiSupport.fullNameOf(rwiNode) + ": added " + addedHyperlinks + " hyperlinks, now has " + countHyperlinks(rwiNode) + " hyperlinks in total.", rwiNode);
        }
        return null;
    }

    private boolean hasHyperlinkTo(RwiElement rwiElement, RwiElement rwiHyperlink) {
        return hasPropertyValue(rwiElement, RwiProperty.HYPERLINK, RwiSupport.uniqueNameOf(rwiHyperlink));
    }
    private boolean hasPropertyValue(RwiElement rwiElement, String propertyName, String value) {
        boolean hasValue = false;
        for (RwiPropertyEnumeration enum = rwiElement.properties(propertyName); enum.hasMoreElements();
            ) {
			RwiProperty property = enum.nextRwiProperty();
            if (property.getValue().equals(value)) {
                hasValue = true;
                break;
            }
        }
        return hasValue;
    }
    private int countHyperlinks(RwiElement rwiElement) {
        return countProperties(rwiElement, RwiProperty.HYPERLINK);
    }
    private int countProperties(RwiElement rwiElement, String propertyName) {
        int count = 0;
        for (RwiPropertyEnumeration enum = rwiElement.properties(propertyName); enum.hasMoreElements();
            enum.nextRwiProperty(), count++) {
        }
        return count;
    }

    public boolean visitMembers() { return false; }
    public Object visitMember(RwiMember rwiMember) {
        return null;
    }

    public boolean visitLinks() { return false; }
    public Object visitLink(RwiLink rwiLink) {
        return null;
    }

    /**
     * Maps rwiNodes (representing top-level classes) to a list of diagrams in which they appear as references
     * (that is, as a shortcut)
     * @supplierRole nodeDiagramList
     * @clientQualifier node
     * @link aggregationByValue
     * @supplierCardinality 1
     */
    private Map rwiNodeDiagramListMap = new HashMap();
}
