package com.togethersoft.modules.bettersoftwarefaster.packageDependencies;

import com.togethersoft.modules.bettersoftwarefaster.rwisupport.*;
import com.togethersoft.openapi.ide.*;
import com.togethersoft.openapi.ide.message.*;
import com.togethersoft.openapi.rwi.*;
import com.togethersoft.openapi.rwi.enum.*;
import com.togethersoft.openapi.sci.*;
import java.util.*;

/**
 * Iterates over the RWI model, and applies the <<patternInstance>>
 * stereotype to any objects that are instantiated from a class with <<pattern>> stereotype.
 */
public class PackageDependenciesModule extends RwiVisitorSupport implements IdeScript {
    public PackageDependenciesModule() {
        super("Analyse Package Dependencies");
    }

    public void run(IdeContext ideContext) {
        getMessagePage().printMessage("Started.");
        if (!isActiveProject())
            return;

        getMessagePage().printMessage("Inspecting model:");
        getRwiVisitorDispatcher().visitAndDispatch();

        getMessagePage().printMessage("Analysis:");
        int numIssues = 0;

        // first check: look for derived package dependencies that
        // have no explicit counterpart
        // we loop around each package that has derived dependencies...
        for (Iterator dependerIter = packageToDerivedDependedPackagesMap.keySet().iterator(); dependerIter.hasNext(); ) {
            RwiDiagram depender = (RwiDiagram)dependerIter.next();

            List derivedDependedPackageList = (List)packageToDerivedDependedPackagesMap.get(depender);
            List explicitDependedPackageList = (List)packageToExplicitDependedPackagesMap.get(depender);
            if (explicitDependedPackageList == null) {
                explicitDependedPackageList = Collections.EMPTY_LIST;
            }
            // ... for those derived dependencies, loop around each depended package ...
            for (Iterator derivedIter = derivedDependedPackageList.iterator(); derivedIter.hasNext(); ) {
                RwiLinkReference derivedLinkReference = (RwiLinkReference)derivedIter.next();
                RwiDiagram derivedDepended = (RwiDiagram)derivedLinkReference.getLink().getDestination();

                // ... and see if exists in the list of explicit depended packages
                boolean foundInExplicit = false;
                for (Iterator explicitIter = explicitDependedPackageList.iterator(); explicitIter.hasNext(); ) {
                    RwiLinkReference explicitLinkReference = (RwiLinkReference)explicitIter.next();
                    RwiDiagram explicitDepended = (RwiDiagram)explicitLinkReference.getLink().getDestination();

                    if (derivedDepended == explicitDepended) {
                        foundInExplicit = true;
                        break;
                    }
                }
                if (!foundInExplicit) {
                    getMessagePage().printMessage( "" + (++numIssues) + ": " + RwiSupport.nameOf(depender) + ": derived dependency to " +
                        RwiSupport.nameOf(derivedDepended) + " but no explicit dependency.", derivedLinkReference);
                }
            }
        }

        // second check: look for bi-directional dependencies
        // we loop around each package that has derived dependencies...
        for (Iterator dependerIter = packageToDerivedDependedPackagesMap.keySet().iterator();dependerIter.hasNext(); ) {
            RwiDiagram depender = (RwiDiagram)dependerIter.next();
			List derivedDependedPackageList = (List)packageToDerivedDependedPackagesMap.get(depender);

            // ... for those derived dependencies, loop around each depended package ...
            for (Iterator dependedIter = derivedDependedPackageList.iterator(); dependedIter.hasNext(); ) {
                RwiLinkReference dependedLinkReference = (RwiLinkReference)dependedIter.next();
                RwiDiagram depended = (RwiDiagram)dependedLinkReference.getLink().getDestination();

				List dependedsDependedPackageList = (List)packageToDerivedDependedPackagesMap.get(depended);
				if (dependedsDependedPackageList == null) {
					dependedsDependedPackageList = Collections.EMPTY_LIST;
                }

                // ... and see if exists in the list of that depended's own depended packages
				boolean dependedDependsOnDepender = false;
	            for (Iterator dependedsDependedIter = dependedsDependedPackageList.iterator(); dependedsDependedIter.hasNext(); ) {
                    RwiLinkReference dependedsDependedLinkReference = (RwiLinkReference)dependedsDependedIter.next();
                    RwiDiagram dependedsDepended = (RwiDiagram)dependedsDependedLinkReference.getLink().getDestination();
					if (dependedsDepended == depender) {
						dependedDependsOnDepender = true;
                        break;
                    }
                }
                if (dependedDependsOnDepender) {
			        getMessagePage().printMessage(
	                    "" + (++numIssues) + ": " + RwiSupport.nameOf(depender) + " and " + RwiSupport.nameOf(depended) +
                        " have a bi-directional dependency", dependedLinkReference);
                }
			}
		}
		if (numIssues == 0) {
	        getMessagePage().printMessage("No issues found.");
        }

        getMessagePage().setVisible(true);
        getMessagePage().printMessage("Finished.");
    }

    public Object visitPackage(RwiPackage rwiPackage) {
        return null;
    }

    public boolean visitDiagrams() {
        return true;
    }

    public Object visitDiagram(RwiDiagram rwiDiagram) {
        for (RwiDiagramReferenceEnumeration enum = rwiDiagram.diagramReferences(); enum.hasMoreElements(); ) {
            RwiDiagramReference dependerReference = enum.nextRwiDiagramReference();

            RwiDiagram depender = dependerReference.getDiagram();

            // only dealing with class diagrams, so continue onto next if not.
            // in fact, we actually want to process packages, but package icons are
            // actually class diagrams in the RWI and not easy to distinguish from manually
            // created class diagrams.  However, manually class diagrams will just not have
            // any outgoing link references, so we are fine to process both
            if (!depender.getProperty(RwiProperty.SHAPE_TYPE).equals(RwiShapeType.CLASS_DIAGRAM)) {
                continue;
            }

            for (RwiLinkReferenceEnumeration linkReferenceEnum = dependerReference.outgoingLinkReferences();linkReferenceEnum.hasMoreElements(); ) {
                RwiLinkReference linkReference = linkReferenceEnum.nextRwiLinkReference();
                RwiLink rwiLink = linkReference.getLink();

                // only dealing with dependency links
                if (!rwiLink.getProperty(RwiProperty.SHAPE_TYPE).equals("DependencyLink")) {
                    continue;
                }

                boolean automatic = rwiLink.hasProperty("automatic");
                List dependedPackagesList;
                if (automatic) {
                    dependedPackagesList = (List)packageToDerivedDependedPackagesMap.get(depender);
                    if (dependedPackagesList == null) {
                        dependedPackagesList = new LinkedList();
                        packageToDerivedDependedPackagesMap.put(depender, dependedPackagesList);
                    }
                } else {
                    dependedPackagesList = (List)packageToExplicitDependedPackagesMap.get(depender);
                    if (dependedPackagesList == null) {
                        dependedPackagesList = new LinkedList();
                        packageToExplicitDependedPackagesMap.put(depender, dependedPackagesList);
                    }
                }
                RwiElement destination = rwiLink.getDestination();
                if (!(destination instanceof RwiDiagram)) {
                    break;
                }
                RwiDiagram depended = (RwiDiagram)destination;
                getMessagePage().printMessage(
                    "- found " + (automatic ? "derived" : "explicit") +
                    " link from " + RwiSupport.nameOf(depender) + " to " +
                    RwiSupport.nameOf(destination), linkReference);
                dependedPackagesList.add(linkReference);
            }
        }
        return null;
    }

    /**
     * Maps each package to list of packages upon which it depends; these dependencies have been explicitly added by the architect
     * <p>Note that we actually key on the RwiDiagram that represents
     * the depender package.  Also, we store a linkReference to an RwiDiagram that
     * represents the depended package (so that we can highlight the dependency)
     * @link aggregationByValue
     * @supplierCardinality 1
     * @supplierRole explicitDependedPackages
     * @clientQualifier package
     */
    private Map packageToExplicitDependedPackagesMap = new HashMap();

    /**
     * Maps each package to list of packages upon which it depends; these dependencies have been derived from code using
     * "Update Package Dependencies". <p>Note that we actually key on the RwiDiagram that represents
     * the depender package.  Also, we store a linkReference to an RwiDiagram that
     * represents the depended package (so that we can highlight the dependency)
     * @link aggregationByValue
     * @supplierCardinality 1
     * @supplierRole derivedDependedPackages
     * @clientQualifier package
     */
    private Map packageToDerivedDependedPackagesMap = new HashMap();

    public boolean visitNodes() {
        return false;
    }

    public Object visitNode(RwiNode rwiNode) {
        return null;
    }

    public boolean visitMembers() { return false; }

    public Object visitMember(RwiMember rwiMember) {
        return null;
    }

    public boolean visitLinks() { return false; }

    public Object visitLink(RwiLink rwiLink) {
        return null;
    }
}
