package com.togethersoft.modules.bettersoftwarefaster.rwisupport;

import com.togethersoft.openapi.ide.*;
import com.togethersoft.openapi.ide.config.*;
import com.togethersoft.openapi.ide.project.*;
import com.togethersoft.openapi.ide.message.*;
import com.togethersoft.openapi.ide.window.*;

import com.togethersoft.openapi.ide.*;
import com.togethersoft.openapi.rwi.*;
import com.togethersoft.openapi.rwi.enum.*;

import java.awt.Component;

import java.util.*;


/**
* Iterates over the RWI model, dispatching to the specified visitor provided
* that the corresponding visitorFilter indicates that this type of node
* (diagram, node, member, outgoing link) should be dispatched to.  Note that
* packages are always dispatched to; the filter interface defines no method
* and is not queried.
*/
public class RwiVisitorDispatcher {

    /**
     * @param rwiVisitor        visitor to be dispatched to elements in the RWI Model
     * @param rwiVisitorFilter  filter for the elements that the visitor should be dispatched to
     */
    public RwiVisitorDispatcher(
        RwiVisitor rwiVisitor,
        RwiVisitorFilter rwiVisitorFilter) {

        this.rwiVisitor = rwiVisitor;
        this.rwiVisitorFilter = rwiVisitorFilter;
    }

	private RwiVisitor rwiVisitor;
	private RwiVisitorFilter rwiVisitorFilter;


	/**
	 * Public interface for this object.
     */
    public void visitAndDispatch() {
        processRwiPackages();
    }

  	////////////////////////////////////////////////////////////////////



	/**
	 * Process all packages.  Subpackage handling is performed by support method.
     */
	 private void processRwiPackages() {

		RwiPackage eachRwiPackage;
		RwiPackageEnumeration enum = RwiSupport.getModelRootPackages();
		for ( ; enum.hasMoreElements(); ) {
 			processRwiPackageAndSubpackages(enum.nextRwiPackage() /*, 0 */);
		}
	}

	/**
     * This method recursively performs the same action upon the RwiPackage
     * passed to it as an argument and upon all its subpackages.
     *
     * @param rwiPackage specified RwiPackage object
	 */
	private void processRwiPackageAndSubpackages(RwiPackage rwiPackage)
    {
		processRwiPackage(rwiPackage /*, depth */);

		RwiPackageEnumeration subPackages = rwiPackage.subpackages();
		RwiPackage eachPackage;
		while (subPackages.hasMoreElements()) {
			eachPackage = subPackages.nextRwiPackage();
			processRwiPackageAndSubpackages(eachPackage /*, depth + 1 */);
        }
    }

	/**
	 * Give the package to the visitor to collect.  Note that the RwiVisitorFilter
     * is <b>not</b> consulted - packages are always iterated over.
     * Then, process diagrams for the package.
     * Then, process nodes for the package.
     * Then, process links for the package (as an element).
	 *
	 * @param rwiNode specified RwiNode object
     * */
	private void processRwiPackage(RwiPackage rwiPackage /*, int depth */) {
		rwiPackage.accept(this.rwiVisitor);
		processRwiDiagramsInPackage(rwiPackage);
		processRwiNodesInPackage(rwiPackage);// traverse nodes in package
        processRwiLinksForElement(rwiPackage);
	}


  	////////////////////////////////////////////////////////////////////

	/*
	 * @param    rwiPackage the package being processed
	 */
	private void processRwiDiagramsInPackage(RwiPackage rwiPackage) {
		RwiDiagramEnumeration enum = rwiPackage.diagrams();
		for ( ; enum.hasMoreElements(); ) {
			processRwiDiagram(enum.nextRwiDiagram());
		}
	}

	/**
	 * Give the diagram to the visitor (if filter allows it).
     * Then, process links for the diagram (as an element).
	 *
	 * @param rwiDiagram specified RwiDiagram object
     */
	private void processRwiDiagram(RwiDiagram rwiDiagram) {
		if (rwiVisitorFilter.visitDiagrams())
			rwiDiagram.accept(rwiVisitor);
        processRwiLinksForElement(rwiDiagram);
	}


	////////////////////////////////////////////////////////////////////


	/*
	 * @param    rwiPackage the package being processed
	 * @param    depth      the depth of subnodes
	 *                      (top level node in package has depth==0)
	 */
	private void processRwiNodesInPackage(RwiPackage rwiPackage) {
		RwiNodeEnumeration enum = rwiPackage.nodes();
		for ( ; enum.hasMoreElements(); ) {
			processRwiNodeAndSubnodes(enum.nextRwiNode(), 0);
		}
	}

	private void processRwiNodeAndSubnodes(RwiNode rwiNode, int depth) {

		processRwiNode(rwiNode, depth);

		RwiNodeEnumeration enum = rwiNode.subnodes();
		RwiNode eachRwiNode;
		for ( ; enum.hasMoreElements(); ) {
			eachRwiNode = enum.nextRwiNode();
			processRwiNodeAndSubnodes(eachRwiNode, depth + 1);
		}
	}

	/**
	 * Give the node to the visitor (if filter allows it).
     * Then, process links for the node (as an element).
	 *
	 * @param rwiNode specified RwiNode object
     * @param depth "depth" of package level
     */
	private void processRwiNode(RwiNode rwiNode, int depth) {

		if (rwiVisitorFilter.visitNodes())
			rwiNode.accept(rwiVisitor);
		processRwiMembersInNode(rwiNode);
        processRwiLinksForElement(rwiNode);
	}

	////////////////////////////////////////////////////////////////////

    private void processRwiMembersInNode(RwiNode rwiNode) {
		RwiMemberEnumeration enum = rwiNode.members();
		for (; enum.hasMoreElements(); ){
			processRwiMember(enum.nextRwiMember());
        }
    }

    private void processRwiMember(RwiMember rwiMember) {
		if (rwiVisitorFilter.visitMembers())
			rwiMember.accept(rwiVisitor);
        processRwiLinksForElement(rwiMember);
    }

  	////////////////////////////////////////////////////////////////////

    private void processRwiLinksForElement(RwiElement rwiElement) {
        RwiLinkEnumeration enum = rwiElement.outgoingLinks();
		for (; enum.hasMoreElements(); ) {
            processRwiLink(enum.nextRwiLink());
        }
    }

    private void processRwiLink(RwiLink rwiLink) {
		if (rwiVisitorFilter.visitLinks())
			rwiLink.accept(rwiVisitor);
    }

}


