package com.togethersoft.modules.inspector.bettersoftwarefaster.inspectorsupport;

import com.togethersoft.openapi.ide.*;
import com.togethersoft.openapi.ide.inspector.*;
import com.togethersoft.openapi.ide.inspector.util.*;
import com.togethersoft.openapi.ide.inspector.util.editors.*;
import com.togethersoft.openapi.ide.inspector.util.property.*;
import com.togethersoft.openapi.ide.inspector.util.table.*;
import com.togethersoft.openapi.ide.util.*;
import com.togethersoft.openapi.rwi.*;
import com.togethersoft.openapi.util.RwiElementsUtil;
import javax.swing.DefaultComboBoxModel;
import java.awt.Color;

/** @represents implementation */
public abstract class IdeInspectorBuilderAdapter implements IdeStartup, IdeInspectorBuilder {
    // debug support
    private Logger logger = new DebugLogger();

    ///////////////////////////////////////////////////////////////////////
    abstract class Logger {
        abstract void log(String message);
    }


    class DebugLogger extends Logger {
        void log(String message) {
            System.out.println(message);
        }
    }


    class NullLogger extends Logger {
        void log(String message) { }
    }


    ///////////////////////////////////////////////////////////////////////
    public void autorun() {
        IdeInspectorManager manager = IdeInspectorManagerAccess.getInspectorManager();
        manager.addInspectorBuilder(this);
    }
    ///////////////////////////////////////////////////////////////////////

    /** template pattern. */
    public IdeInspector buildInspector(IdeContext context, IdeInspector inspector) {

        // save away selected elements.
        // it's safe to do this - we're synchronized, and
        // buildInspector is called whenever the selection changes.
        this.context = context;
        this.inspector = inspector;
        this.rwiElements = RwiElementsUtil.getRwiElements(context);
        if (isSelectedRwiElementClassDiagram()) {
            logger.log("*** building class diagram inspector");
            buildClassDiagramInspector();
        }
        else if (isSelectedRwiElementClass()) {
            logger.log("*** building class inspector");
            buildClassInspector();
        }
        else if (isSelectedRwiElementInterface()) {
            logger.log("*** building interface inspector");
            buildInterfaceInspector();
        }
        else if (isSelectedRwiElementAttribute()) {
            logger.log("*** building attribute inspector");
            buildAttributeInspector();
        }
        else if (isSelectedRwiElementLink()) {
            logger.log("*** building link inspector");
            buildLinkInspector();
        }
        else if (isSelectedRwiElementOperation()) {
            logger.log("*** building operation inspector");
            buildOperationInspector();
        }
        return inspector;
    }

    private IdeContext context;
    private IdeInspector inspector;
    private RwiElement[] rwiElements;

    ///////////////////////////////////////////////////////////////////////
    protected void buildClassDiagramInspector() { }

    protected void buildClassInspector() { }

    protected void buildInterfaceInspector() { }

    protected void buildAttributeInspector() { }

    protected void buildLinkInspector() { }

    protected void buildOperationInspector() { }

    ///////////////////////////////////////////////////////////////////////
    protected IdeContext getIdeContext() { return context; }

    protected IdeInspector getIdeInspector() { return inspector; }

    protected RwiElement[] getRwiElements() { return rwiElements; }

    ///////////////////////////////////////////////////////////////////////
    protected void addStringProperty(String pageName, String propertyName, String uiName) {
        addStringProperty(pageName, propertyName, uiName, null);
    }

    protected void addStringProperty(String pageName, String propertyName, String uiName, Condition condition) {
        IdeInspectorProperty property = new RwiInspectorStringProperty(this.rwiElements, propertyName);
        property.setName(uiName);
        findOrCreatePage(pageName).addProperty(property, condition);
    }

    protected void addBooleanProperty(String pageName, String propertyName, String uiName) {
        addBooleanProperty(pageName, propertyName, uiName, null);
    }

    protected void addBooleanProperty(String pageName, String propertyName, String uiName, Condition condition) {
        IdeInspectorProperty property = new RwiInspectorBooleanProperty(this.rwiElements, propertyName);
        property.setName(uiName);
        // add to page, (shown as a checkbox in the UI)
        findOrCreatePage(pageName).addProperty(property, condition);
    }

    protected void addIntegerProperty(String pageName, String propertyName, String uiName) {
        addIntegerProperty(pageName, propertyName, uiName, null);
    }

    protected void addIntegerProperty(String pageName, String propertyName, String uiName, Condition condition) {
        IdeInspectorProperty property = new RwiInspectorIntegerProperty(this.rwiElements, propertyName);
        property.setName(uiName);
        findOrCreatePage(pageName).addProperty(property, condition);
    }

    protected void addFloatProperty(String pageName, String propertyName, String uiName) {
        addFloatProperty(pageName, propertyName, uiName, null);
    }

    protected void addFloatProperty(String pageName, String propertyName, String uiName, Condition condition) {
        IdeInspectorProperty property = new RwiInspectorStringProperty(this.rwiElements, propertyName);
        property.setName(uiName);
        SwingNumberEditor editor = new SwingNumberEditor();
        property.setPropertyEditor(editor);
        findOrCreatePage(pageName).addProperty(property, condition);
    }

    protected void addChoiceProperty(String pageName, String propertyName, String uiName, String[] choices) {
        addChoiceProperty(pageName, propertyName, uiName, choices, null);
    }

    protected void addChoiceProperty(String pageName, String propertyName, String uiName, String[] choices, Condition condition) {
        addChoiceComboBoxProperty(pageName, propertyName, uiName, choices, condition, false);
    }

    protected void addComboBoxProperty(String pageName, String propertyName, String uiName, String[] choices) {
        addComboBoxProperty(pageName, propertyName, uiName, choices, null);
    }

    protected void addComboBoxProperty(String pageName, String propertyName, String uiName, String[] choices, Condition condition) {
        addChoiceComboBoxProperty(pageName, propertyName, uiName, choices, condition, true);
    }

    // editable = true is combobox; = false is a choice field
    private void addChoiceComboBoxProperty(String pageName, String propertyName, String uiName, String[] choices, Condition condition, boolean editable) {
        IdeInspectorProperty property = new RwiInspectorStringProperty(this.rwiElements, propertyName);
        property.setName(uiName);
        SwingComboBoxEditor editor = new SwingComboBoxEditor(new DefaultComboBoxModel(choices));
        property.setPropertyEditor(editor);
        editor.setValueEditable(editable);
        findOrCreatePage(pageName).addProperty(property, condition);
    }

    ///////////////////////////////////////////////////////////////////////
    

    protected void addExtendedStringProperty(String pageName, String propertyName, String uiName) {
        addExtendedStringProperty(pageName, propertyName, uiName, null);
    }

    protected void addExtendedStringProperty(String pageName, String propertyName, String uiName, Condition condition) {
        IdeInspectorProperty property = new RwiInspectorStringProperty(this.rwiElements, propertyName);
        property.setName(uiName);
        ExtendedStringEditor editor = new ExtendedStringEditor(uiName);
        property.setPropertyEditor(editor);
        findOrCreatePage(pageName).addProperty(property, condition);
    }

    protected void addClassProperty(String pageName, String propertyName, String uiName, int ideSelectorDialogType) {
        addClassProperty(pageName, propertyName, uiName, ideSelectorDialogType, null);
    }

    protected void addClassProperty(String pageName, String propertyName, String uiName, int ideSelectorDialogType, Condition condition) {
        IdeInspectorProperty property = new RwiInspectorStringProperty(this.rwiElements, propertyName);
        property.setName(uiName);
        SingleElementChooserEditor editor = new SingleElementChooserEditor(this.rwiElements[0], ideSelectorDialogType);
        property.setPropertyEditor(editor);
        findOrCreatePage(pageName).addProperty(property, condition);
    }

    protected void addMultipleStringProperty(String pageName, String propertyName, String uiName) {
        addMultipleStringProperty(pageName, propertyName, uiName, null);
    }

    protected void addMultipleStringProperty(String pageName, String propertyName, String uiName, Condition condition) {
            IdeInspectorProperty property = new RwiInspectorMultipleStringProperty(rwiElements, propertyName);
            property.setName(propertyName);
            findOrCreatePage(pageName).addProperty(property, condition);
        }

    protected void addURLProperty(String pageName, String propertyName, String uiName) {
        addURLProperty(pageName, propertyName, uiName, null);
    }

    protected void addURLProperty(String pageName, String propertyName, String uiName, Condition condition) {
        IdeInspectorProperty property = new RwiInspectorStringProperty(this.rwiElements, propertyName);
        property.setName(uiName);
        SwingURLEditor editor = new SwingURLEditor();
        property.setPropertyEditor(editor);
        findOrCreatePage(pageName).addProperty(property, condition);
    }

    protected void addInversedBooleanProperty(String pageName, String propertyName, String uiName) {
		addInversedBooleanProperty(pageName, propertyName, uiName, null);
    }

    protected void addInversedBooleanProperty(String pageName, String propertyName, String uiName, Condition condition) {
        IdeInspectorProperty property = new RwiInspectorInversedBooleanProperty(this.rwiElements, propertyName);
        property.setName(uiName);
        findOrCreatePage(pageName).addProperty(property, condition);
    }

    protected void addColorProperty(String pageName, String propertyName, String uiName) {
        addColorProperty(pageName, propertyName, uiName, null);
    }

    protected void addColorProperty(String pageName, String propertyName, String uiName, Condition condition) {
        IdeInspectorProperty property = new RwiInspectorColorProperty(this.rwiElements, propertyName);
        property.setName(uiName);
        property.setPropertyEditor( new SwingColorEditor( Color.black, uiName ) );
        findOrCreatePage(pageName).addProperty(property, condition);
    }

    private IdeInspectorPropertySetComponent findOrCreatePage(String componentPageName) {

        logger.log("findOrCreatePage: " + componentPageName);

        IdeInspectorComponent[] components = this.inspector.getComponents();
		for (int i=0; i<components.length;i++) {
            logger.log("  inspector has a IdeInspectorComponent: " + components[i].getName() );
        }

        IdeInspectorComponent component = this.inspector.findComponent(componentPageName);
        IdeInspectorPropertySetComponent page = null;
        if (component instanceof IdeInspectorPropertySetComponent) {
            logger.log("found page: " + componentPageName);
            page = (IdeInspectorPropertySetComponent)component;
        }
        else {
            logger.log("creating page: " + componentPageName);
            page = new IdeInspectorPropertySetComponent(this.context);
            page.setName(componentPageName);
            page.setService(UIComponentService.class, new PropertyTableComponentUI(page));
            this.inspector.addComponent(page, null);
        }
        return page;
    }

    ///////////////////////////////////////////////////////////////////////
    private boolean isSelectedRwiElementClassDiagram() {
        return isSelectedRwiElementShapeType(RwiShapeType.CLASS_DIAGRAM);
    }

    private boolean isSelectedRwiElementClassOrInterface() {
        return isSelectedRwiElementShapeType(RwiShapeType.CLASS);
    }

    private boolean isSelectedRwiElementClass() {
        return isSelectedRwiElementClassOrInterface() && !isSelectedRwiElementInterface();
    }

    private boolean isSelectedRwiElementInterface() {
        return isSelectedRwiElementClassOrInterface() && isSelectedRwiElementShapeType(RwiShapeType.INTERFACE);
    }

    private boolean isSelectedRwiElementOperation() {
        return isSelectedRwiElementShapeType(RwiShapeType.OPERATION);
    }

    private boolean isSelectedRwiElementLink() {
        return isSelectedRwiElementShapeType(RwiShapeType.ASSOCIATION);
    }

    private boolean isSelectedRwiElementAttribute() {
        return isSelectedRwiElementShapeType(RwiShapeType.ATTRIBUTE);
    }

    private boolean isSelectedRwiElementShapeType(String shapeType) {
        return RwiElementsUtil.checkProperty(this.context, RwiProperty.SHAPE_TYPE, shapeType);
    }
}
